from typing import Tuple, Set, Iterable, List


class ItemFactoryBase(APIObject):
    def NewDetailCurve(self, view: View, geometryCurve: Curve) -> DetailCurve: ...
    def NewDetailCurveArray(self, view: View, geometryCurveArray: CurveArray) -> DetailCurveArray: ...
    @overload
    def NewDimension(self, view: View, line: Line, references: ReferenceArray, dimensionType: DimensionType) -> Dimension: ...
    @overload
    def NewDimension(self, view: View, line: Line, references: ReferenceArray) -> Dimension: ...
    @overload
    def NewFamilyInstance(self, line: Line, symbol: FamilySymbol, specView: View) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, origin: XYZ, symbol: FamilySymbol, specView: View) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, reference: Reference, position: Line, symbol: FamilySymbol) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, reference: Reference, location: XYZ, referenceDirection: XYZ, symbol: FamilySymbol) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, face: Face, position: Line, symbol: FamilySymbol) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, face: Face, location: XYZ, referenceDirection: XYZ, symbol: FamilySymbol) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, location: XYZ, symbol: FamilySymbol, structuralType: StructuralType) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, location: XYZ, symbol: FamilySymbol, host: Element, structuralType: StructuralType) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, location: XYZ, symbol: FamilySymbol, referenceDirection: XYZ, host: Element, structuralType: StructuralType) -> FamilyInstance: ...
    def NewFamilyInstances2(self, dataList: List) -> ICollection: ...
    def NewGroup(self, elementIds: ICollection) -> Group: ...
    def NewModelCurve(self, geometryCurve: Curve, sketchPlane: SketchPlane) -> ModelCurve: ...
    def NewModelCurveArray(self, geometryCurveArray: CurveArray, sketchPlane: SketchPlane) -> ModelCurveArray: ...
    def NewReferencePlane(self, bubbleEnd: XYZ, freeEnd: XYZ, cutVec: XYZ, pView: View) -> ReferencePlane: ...
    def NewReferencePlane2(self, bubbleEnd: XYZ, freeEnd: XYZ, thirdPnt: XYZ, pView: View) -> ReferencePlane: ...
    def PlaceGroup(self, location: XYZ, groupType: GroupType) -> Group: ...
    def NewAlignment(self, view: View, reference1: Reference, reference2: Reference) -> Dimension: ...


class FamilyItemFactory(ItemFactoryBase):
    def NewExtrusion(self, isSolid: bool, profile: CurveArrArray, sketchPlane: SketchPlane, end: float) -> Extrusion: ...
    def NewBlend(self, isSolid: bool, profile1: CurveArray, profile2: CurveArray, sketchPlane: SketchPlane) -> Blend: ...
    def NewRevolution(self, isSolid: bool, profile: CurveArrArray, sketchPlane: SketchPlane, axis: Line, startAngle: float, endAngle: float) -> Revolution: ...
    @overload
    def NewSweep(self, isSolid: bool, path: ReferenceArray, profile: SweepProfile, profileLocationCurveIndex: int, profilePlaneLocation: ProfilePlaneLocation) -> Sweep: ...
    @overload
    def NewSweep(self, isSolid: bool, path: CurveArray, pathPlane: SketchPlane, profile: SweepProfile, profileLocationCurveIndex: int, profilePlaneLocation: ProfilePlaneLocation) -> Sweep: ...
    @overload
    def NewSweptBlend(self, isSolid: bool, path: Reference, bottomProfile: SweepProfile, topProfile: SweepProfile) -> SweptBlend: ...
    @overload
    def NewSweptBlend(self, isSolid: bool, path: Curve, pathPlane: SketchPlane, bottomProfile: SweepProfile, topProfile: SweepProfile) -> SweptBlend: ...
    def NewLoftForm(self, isSolid: bool, profiles: ReferenceArrayArray) -> Form: ...
    def NewExtrusionForm(self, isSolid: bool, profile: ReferenceArray, direction: XYZ) -> Form: ...
    def NewSweptBlendForm(self, isSolid: bool, path: ReferenceArray, profiles: ReferenceArrayArray) -> Form: ...
    def NewRevolveForms(self, isSolid: bool, profile: ReferenceArray, axis: Reference, startAngle: float, endAngle: float) -> FormArray: ...
    def NewFormByCap(self, isSolid: bool, profile: ReferenceArray) -> Form: ...
    def NewFormByThickenSingleSurface(self, isSolid: bool, singleSurfaceForm: Form, thickenDir: XYZ) -> Form: ...
    @overload
    def NewLinearDimension(self, view: View, line: Line, references: ReferenceArray, dimensionType: DimensionType) -> Dimension: ...
    @overload
    def NewLinearDimension(self, view: View, line: Line, references: ReferenceArray) -> Dimension: ...
    @overload
    def NewAngularDimension(self, view: View, arc: Arc, firstRef: Reference, secondRef: Reference, dimensionType: DimensionType) -> Dimension: ...
    @overload
    def NewAngularDimension(self, view: View, arc: Arc, firstRef: Reference, secondRef: Reference) -> Dimension: ...
    @overload
    def NewArcLengthDimension(self, view: View, arc: Arc, arcRef: Reference, firstRef: Reference, secondRef: Reference, dimensionType: DimensionType) -> Dimension: ...
    @overload
    def NewArcLengthDimension(self, view: View, arc: Arc, arcRef: Reference, firstRef: Reference, secondRef: Reference) -> Dimension: ...
    @overload
    def NewRadialDimension(self, view: View, arcRef: Reference, origin: XYZ, dimensionType: DimensionType) -> Dimension: ...
    @overload
    def NewRadialDimension(self, view: View, arcRef: Reference, origin: XYZ) -> Dimension: ...
    def NewDiameterDimension(self, view: View, arcRef: Reference, origin: XYZ) -> Dimension: ...
    def NewOpening(self, host: Element, profile: CurveArray) -> Opening: ...
    def NewModelText(self, text: str, modelTextType: ModelTextType, sketchPlane: SketchPlane, position: XYZ, horizontalAlign: HorizontalAlign, depth: float) -> ModelText: ...
    def NewControl(self, controlShape: ControlShape, view: View, origin: XYZ) -> Control: ...
    def NewSymbolicCurve(self, curve: Curve, sketchPlane: SketchPlane) -> SymbolicCurve: ...
    @overload
    def NewReferencePoint(self, A_0: PointElementReference) -> ReferencePoint: ...
    @overload
    def NewReferencePoint(self, A_0: Transform) -> ReferencePoint: ...
    @overload
    def NewReferencePoint(self, A_0: XYZ) -> ReferencePoint: ...
    def NewCurveByPoints(self, points: ReferencePointArray) -> CurveByPoints: ...


class Application(APIObject):
    def NewCategorySet(self) -> CategorySet: ...
    @overload
    def NewInstanceBinding(self, categorySet: CategorySet) -> InstanceBinding: ...
    @overload
    def NewInstanceBinding(self) -> InstanceBinding: ...
    @overload
    def NewTypeBinding(self, categorySet: CategorySet) -> TypeBinding: ...
    @overload
    def NewTypeBinding(self) -> TypeBinding: ...
    def NewElementSet(self) -> ElementSet: ...
    def NewGeometryOptions(self) -> Options: ...
    def NewCurveArray(self) -> CurveArray: ...
    def NewCurveArrArray(self) -> CurveArrArray: ...
    def NewVertexIndexPair(self, iTop: int, iBottom: int) -> VertexIndexPair: ...
    def NewVertexIndexPairArray(self) -> VertexIndexPairArray: ...
    def NewCombinableElementArray(self) -> CombinableElementArray: ...
    def NewColor(self) -> Color: ...
    def NewProjectPosition(self, ew: float, ns: float, elevation: float, angle: float) -> ProjectPosition: ...
    def NewBoundingBoxXYZ(self) -> BoundingBoxXYZ: ...
    @overload
    def NewBoundingBoxUV(self, min_u: float, min_v: float, max_u: float, max_v: float) -> BoundingBoxUV: ...
    @overload
    def NewBoundingBoxUV(self) -> BoundingBoxUV: ...
    @overload
    def NewXYZ(self, xyz: XYZ) -> XYZ: ...
    @overload
    def NewXYZ(self, x: float, y: float, z: float) -> XYZ: ...
    @overload
    def NewXYZ(self) -> XYZ: ...
    @overload
    def NewUV(self, uv: UV) -> UV: ...
    @overload
    def NewUV(self, u: float, v: float) -> UV: ...
    @overload
    def NewUV(self) -> UV: ...
    def NewDWFExportOptions(self) -> DWFExportOptions: ...
    def NewDWFXExportOptions(self) -> DWFXExportOptions: ...
    def NewFBXExportOptions(self) -> FBXExportOptions: ...
    def NewGBXMLImportOptions(self) -> GBXMLImportOptions: ...
    def NewDoubleArray(self) -> DoubleArray: ...
    def NewReferenceArray(self) -> ReferenceArray: ...
    def NewFaceArray(self) -> FaceArray: ...
    def NewIntersectionResultArray(self) -> IntersectionResultArray: ...
    def NewViewSet(self) -> ViewSet: ...
    def NewSpaceSet(self) -> SpaceSet: ...
    @overload
    def NewFamilyInstanceCreationData(self, symbol: FamilySymbol, adaptivePoints: List[XYZ]) -> FamilyInstanceCreationData: ...
    @overload
    def NewFamilyInstanceCreationData(self, face: Face, position: Line, symbol: FamilySymbol) -> FamilyInstanceCreationData: ...
    @overload
    def NewFamilyInstanceCreationData(self, face: Face, location: XYZ, referenceDirection: XYZ, symbol: FamilySymbol) -> FamilyInstanceCreationData: ...
    @overload
    def NewFamilyInstanceCreationData(self, location: XYZ, symbol: FamilySymbol, referenceDirection: XYZ, host: Element, structuralType: StructuralType) -> FamilyInstanceCreationData: ...
    @overload
    def NewFamilyInstanceCreationData(self, location: XYZ, symbol: FamilySymbol, host: Element, level: Level, structuralType: StructuralType) -> FamilyInstanceCreationData: ...
    @overload
    def NewFamilyInstanceCreationData(self, location: XYZ, symbol: FamilySymbol, host: Element, structuralType: StructuralType) -> FamilyInstanceCreationData: ...
    @overload
    def NewFamilyInstanceCreationData(self, location: XYZ, symbol: FamilySymbol, level: Level, structuralType: StructuralType) -> FamilyInstanceCreationData: ...
    @overload
    def NewFamilyInstanceCreationData(self, curve: Curve, symbol: FamilySymbol, level: Level, structuralType: StructuralType) -> FamilyInstanceCreationData: ...
    @overload
    def NewFamilyInstanceCreationData(self, location: XYZ, symbol: FamilySymbol, structuralType: StructuralType) -> FamilyInstanceCreationData: ...
    def NewAreaCreationData(self, areaView: ViewPlan, point: UV) -> AreaCreationData: ...
    def NewElementId(self) -> ElementId: ...
    def NewCurveLoopsProfile(self, curveLoops: CurveArrArray) -> CurveLoopsProfile: ...
    def NewFamilySymbolProfile(self, familySymbol: FamilySymbol) -> FamilySymbolProfile: ...
    def NewPointOnEdge(self, edgeReference: Reference, locationOnCurve: PointLocationOnCurve) -> PointOnEdge: ...
    def NewPointOnPlane(self, planeReference: Reference, position: UV, xvec: UV, offset: float) -> PointOnPlane: ...
    def NewPointOnFace(self, faceReference: Reference, uv: UV) -> PointOnFace: ...
    def NewPointOnEdgeEdgeIntersection(self, edgeReference1: Reference, edgeReference2: Reference) -> PointOnEdgeEdgeIntersection: ...
    def NewPointOnEdgeFaceIntersection(self, edgeReference: Reference, faceReference: Reference, orientWithEdge: bool) -> PointOnEdgeFaceIntersection: ...
    def NewPointRelativeToPoint(self, hostPointReference: Reference) -> PointRelativeToPoint: ...
    def NewReferencePointArray(self) -> ReferencePointArray: ...


class FamilyInstanceCreationData:
    @overload
    def __init__(self, proxy: Object): ...
    @overload
    def __init__(self, symbol: FamilySymbol, adaptivePoints: List[XYZ]): ...
    @overload
    def __init__(self, face: Face, position: Line, symbol: FamilySymbol): ...
    @overload
    def __init__(self, face: Face, location: XYZ, referenceDirection: XYZ, symbol: FamilySymbol): ...
    @overload
    def __init__(self, location: XYZ, symbol: FamilySymbol, referenceDirection: XYZ, host: Element, structuralType: StructuralType): ...
    @overload
    def __init__(self, location: XYZ, symbol: FamilySymbol, host: Element, level: Level, structuralType: StructuralType): ...
    @overload
    def __init__(self, location: XYZ, symbol: FamilySymbol, host: Element, structuralType: StructuralType): ...
    @overload
    def __init__(self, location: XYZ, symbol: FamilySymbol, level: Level, structuralType: StructuralType): ...
    @overload
    def __init__(self, curve: Curve, symbol: FamilySymbol, level: Level, structuralType: StructuralType): ...
    @overload
    def __init__(self, location: XYZ, symbol: FamilySymbol, structuralType: StructuralType): ...
    @property
    def Axis(self) -> Line: ...
    @Axis.setter
    def Axis(self, value: Line) -> None: ...
    @property
    def RotateAngle(self) -> float: ...
    @RotateAngle.setter
    def RotateAngle(self, value: float) -> None: ...


class AreaCreationData:
    @overload
    def __init__(self, proxy: Object): ...
    @overload
    def __init__(self, areaView: ViewPlan, point: UV): ...
    @property
    def TagPoint(self) -> UV: ...
    @TagPoint.setter
    def TagPoint(self, value: UV) -> None: ...


class Document(ItemFactoryBase):
    def NewSpotCoordinate(self, view: View, reference: Reference, origin: XYZ, bend: XYZ, end: XYZ, refPt: XYZ, hasLeader: bool) -> SpotDimension: ...
    def NewSpotElevation(self, view: View, reference: Reference, origin: XYZ, bend: XYZ, end: XYZ, refPt: XYZ, hasLeader: bool) -> SpotDimension: ...
    @overload
    def NewFloor(self, profile: CurveArray, floorType: FloorType, level: Level, structural: bool, normal: XYZ) -> Floor: ...
    @overload
    def NewFloor(self, profile: CurveArray, floorType: FloorType, level: Level, structural: bool) -> Floor: ...
    @overload
    def NewFloor(self, profile: CurveArray, structural: bool) -> Floor: ...
    def NewFoundationSlab(self, profile: CurveArray, floorType: FloorType, level: Level, structural: bool, normal: XYZ) -> Floor: ...
    @overload
    def NewRoom(self, room: Room, circuit: PlanCircuit) -> Room: ...
    @overload
    def NewRoom(self, phase: Phase) -> Room: ...
    @overload
    def NewRoom(self, level: Level, point: UV) -> Room: ...
    @overload
    def NewRooms2(self, phase: Phase, count: int) -> ICollection: ...
    @overload
    def NewRooms2(self, level: Level, phase: Phase) -> ICollection: ...
    @overload
    def NewRooms2(self, level: Level) -> ICollection: ...
    def NewRoomTag(self, roomId: LinkElementId, point: UV, viewId: ElementId) -> RoomTag: ...
    def NewPointBoundaryConditions(self, reference: Reference, X_Translation: TranslationRotationValue, X_TranslationSpringModulus: float, Y_Translation: TranslationRotationValue, Y_TranslationSpringModulus: float, Z_Translation: TranslationRotationValue, Z_TranslationSpringModulus: float, X_Rotation: TranslationRotationValue, X_RotationSpringModulus: float, Y_Rotation: TranslationRotationValue, Y_RotationSpringModulus: float, Z_Rotation: TranslationRotationValue, Z_RotationSpringModulus: float) -> BoundaryConditions: ...
    @overload
    def NewLineBoundaryConditions(self, hostElement: Element, X_Translation: TranslationRotationValue, X_TranslationSpringModulus: float, Y_Translation: TranslationRotationValue, Y_TranslationSpringModulus: float, Z_Translation: TranslationRotationValue, Z_TranslationSpringModulus: float, X_Rotation: TranslationRotationValue, X_RotationSpringModulus: float) -> BoundaryConditions: ...
    @overload
    def NewLineBoundaryConditions(self, reference: Reference, X_Translation: TranslationRotationValue, X_TranslationSpringModulus: float, Y_Translation: TranslationRotationValue, Y_TranslationSpringModulus: float, Z_Translation: TranslationRotationValue, Z_TranslationSpringModulus: float, X_Rotation: TranslationRotationValue, X_RotationSpringModulus: float) -> BoundaryConditions: ...
    @overload
    def NewAreaBoundaryConditions(self, hostElement: Element, X_Translation: TranslationRotationValue, X_TranslationSpringModulus: float, Y_Translation: TranslationRotationValue, Y_TranslationSpringModulus: float, Z_Translation: TranslationRotationValue, Z_TranslationSpringModulus: float) -> BoundaryConditions: ...
    @overload
    def NewAreaBoundaryConditions(self, reference: Reference, X_Translation: TranslationRotationValue, X_TranslationSpringModulus: float, Y_Translation: TranslationRotationValue, Y_TranslationSpringModulus: float, Z_Translation: TranslationRotationValue, Z_TranslationSpringModulus: float) -> BoundaryConditions: ...
    @overload
    def NewOpening(self, hostElement: Element, profile: CurveArray, bPerpendicularFace: bool) -> Opening: ...
    @overload
    def NewOpening(self, wall: Wall, pntStart: XYZ, pntEnd: XYZ) -> Opening: ...
    @overload
    def NewOpening(self, bottomLevel: Level, topLevel: Level, profile: CurveArray) -> Opening: ...
    @overload
    def NewOpening(self, famInstElement: Element, profile: CurveArray, iFace: eRefFace) -> Opening: ...
    def NewSlab(self, profile: CurveArray, level: Level, slopedArrow: Line, slope: float, isStructural: bool) -> Floor: ...
    def NewAreaBoundaryLine(self, sketchPlane: SketchPlane, geometryCurve: Curve, areaView: ViewPlan) -> ModelCurve: ...
    def NewAreaTag(self, areaView: ViewPlan, room: Area, point: UV) -> AreaTag: ...
    def NewArea(self, areaView: ViewPlan, point: UV) -> Area: ...
    def NewAreas(self, dataList: List) -> ElementSet: ...
    def NewFootPrintRoof(self, footPrint: CurveArray, level: Level, roofType: RoofType) -> Tuple[FootPrintRoof, ModelCurveArray]: ...
    def NewExtrusionRoof(self, profile: CurveArray, refPlane: ReferencePlane, level: Level, roofType: RoofType, extrusionStart: float, extrusionEnd: float) -> ExtrusionRoof: ...
    def NewMechanicalSystem(self, baseEquipmentConnector: Connector, connectors: ConnectorSet, ductSystemType: DuctSystemType) -> MechanicalSystem: ...
    def NewPipingSystem(self, baseEquipmentConnector: Connector, connectors: ConnectorSet, pipingSystemType: PipeSystemType) -> PipingSystem: ...
    @overload
    def NewSpace(self, level: Level, phase: Phase, point: UV) -> Space: ...
    @overload
    def NewSpace(self, level: Level, point: UV) -> Space: ...
    @overload
    def NewSpace(self, phase: Phase) -> Space: ...
    @overload
    def NewSpaces2(self, phase: Phase, count: int) -> ICollection: ...
    @overload
    def NewSpaces2(self, level: Level, phase: Phase, view: View) -> ICollection: ...
    def NewSpaceTag(self, space: Space, point: UV, view: View) -> SpaceTag: ...
    def NewSpaceBoundaryLines(self, sketchPlane: SketchPlane, curves: CurveArray, view: View) -> ModelCurveArray: ...
    def NewRoomBoundaryLines(self, sketchPlane: SketchPlane, curves: CurveArray, view: View) -> ModelCurveArray: ...
    def NewZone(self, level: Level, phase: Phase) -> Zone: ...
    def NewCurtainSystem2(self, faces: ReferenceArray, curtainSystemType: CurtainSystemType) -> ICollection: ...
    def NewCurtainSystem(self, faces: FaceArray, curtainSystemType: CurtainSystemType) -> CurtainSystem: ...
    @overload
    def NewSlabEdge(self, SlabEdgeType: SlabEdgeType, reference: Reference) -> SlabEdge: ...
    @overload
    def NewSlabEdge(self, SlabEdgeType: SlabEdgeType, references: ReferenceArray) -> SlabEdge: ...
    @overload
    def NewGutter(self, GutterType: GutterType, reference: Reference) -> Gutter: ...
    @overload
    def NewGutter(self, GutterType: GutterType, references: ReferenceArray) -> Gutter: ...
    @overload
    def NewFascia(self, FasciaType: FasciaType, reference: Reference) -> Fascia: ...
    @overload
    def NewFascia(self, FasciaType: FasciaType, references: ReferenceArray) -> Fascia: ...
    @overload
    def NewFamilyInstance(self, curve: Curve, symbol: FamilySymbol, level: Level, structuralType: StructuralType) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, location: XYZ, symbol: FamilySymbol, level: Level, structuralType: StructuralType) -> FamilyInstance: ...
    @overload
    def NewFamilyInstance(self, location: XYZ, symbol: FamilySymbol, host: Element, level: Level, structuralType: StructuralType) -> FamilyInstance: ...
    @overload
    def NewFlexDuct(self, connector1: Connector, connector2: Connector, ductType: FlexDuctType) -> FlexDuct: ...
    @overload
    def NewFlexDuct(self, connector: Connector, points: List[XYZ], ductType: FlexDuctType) -> FlexDuct: ...
    @overload
    def NewFlexDuct(self, points: List[XYZ], ductType: FlexDuctType) -> FlexDuct: ...
    @overload
    def NewFlexPipe(self, connector1: Connector, connector2: Connector, pipeType: FlexPipeType) -> FlexPipe: ...
    @overload
    def NewFlexPipe(self, connector: Connector, points: List[XYZ], pipeType: FlexPipeType) -> FlexPipe: ...
    @overload
    def NewFlexPipe(self, points: List[XYZ], pipeType: FlexPipeType) -> FlexPipe: ...
    def NewElbowFitting(self, connector1: Connector, connector2: Connector) -> FamilyInstance: ...
    def NewTeeFitting(self, connector1: Connector, connector2: Connector, connector3: Connector) -> FamilyInstance: ...
    def NewTransitionFitting(self, connector1: Connector, connector2: Connector) -> FamilyInstance: ...
    def NewCrossFitting(self, connector1: Connector, connector2: Connector, connector3: Connector, connector4: Connector) -> FamilyInstance: ...
    def NewUnionFitting(self, connector1: Connector, connector2: Connector) -> FamilyInstance: ...
    def NewTakeoffFitting(self, connector: Connector, curve: MEPCurve) -> FamilyInstance: ...


class eRefFace:
    CenterX = 1
    CenterY = 4
    CenterZ = 7



"""pyRevit related links."""
from pyrevit.labs import Common

PYREVIT_LICENSE = Common.PyRevitLabsConsts.LicenseUrl
PYREVIT_CREDITS = Common.PyRevitLabsConsts.CreditsUrl
PYREVIT_WIKI = Common.PyRevitLabsConsts.WikiUrl
PYREVIT_TWITTER = "https://twitter.com/pyrevit"
PYREVIT_BLOG = Common.PyRevitLabsConsts.BlogsUrl
PYREVIT_DOCS = Common.PyRevitLabsConsts.DocsUrl
PYREVIT_GITHUB = Common.PyRevitLabsConsts.OriginalRepoBasePath
PYREVIT_GITHUBBRANCH = \
    Common.PyRevitLabsConsts.OriginalRepoBasePath + '/tree/{branch}'
PYREVIT_GITHUBMASTER_COMMIT = \
    Common.PyRevitLabsConsts.OriginalRepoBasePath + '/commits/master'
PYREVIT_GITHUBBRANCH_COMMIT = \
    Common.PyRevitLabsConsts.OriginalRepoBasePath + '/commits/{branch}'
PYREVIT_GITHUBISSUES = Common.PyRevitLabsConsts.IssuesUrl
PYREVIT_YOUTUBE = Common.PyRevitLabsConsts.YoutubeUrl
PYREVIT_SUPPORT = Common.PyRevitLabsConsts.SupportUrl
PYREVIT_RELEASENOTES = Common.PyRevitLabsConsts.ReleasesUrl
PROFILE_EIN = 'https://ein.sh'



class VirtualizationCacheLength(object,IEquatable[VirtualizationCacheLength]):
 """
 VirtualizationCacheLength(cacheBeforeViewport: float,cacheAfterViewport: float)

 VirtualizationCacheLength(cacheBeforeAndAfterViewport: float)
 """
 def Equals(self,*__args):
  """
  Equals(self: VirtualizationCacheLength,cacheLength: VirtualizationCacheLength) -> bool

  Equals(self: VirtualizationCacheLength,oCompare: object) -> bool
  """
  pass
 def GetHashCode(self):
  """ GetHashCode(self: VirtualizationCacheLength) -> int """
  pass
 def ToString(self):
  """ ToString(self: VirtualizationCacheLength) -> str """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,cacheBeforeAndAfterViewport: float)

  __new__(cls: type,cacheBeforeViewport: float,cacheAfterViewport: float)

  __new__[VirtualizationCacheLength]() -> VirtualizationCacheLength
  """
  pass
 def __ne__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 CacheAfterViewport=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: CacheAfterViewport(self: VirtualizationCacheLength) -> float



"""

 CacheBeforeViewport=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: CacheBeforeViewport(self: VirtualizationCacheLength) -> float



"""





class SelectionRange(object):
 """
 Represents a date selection range in a month calendar control.

 

 SelectionRange()

 SelectionRange(lower: DateTime,upper: DateTime)

 SelectionRange(range: SelectionRange)
 """
 def ToString(self):
  """
  ToString(self: SelectionRange) -> str

  

   Returns a string that represents the System.Windows.Forms.SelectionRange.

   Returns: A string that represents the current System.Windows.Forms.SelectionRange.
  """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,lower: DateTime,upper: DateTime)

  __new__(cls: type,range: SelectionRange)
  """
  pass
 End=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the ending date and time of the selection range.



Get: End(self: SelectionRange) -> DateTime



Set: End(self: SelectionRange)=value

"""

 Start=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the starting date and time of the selection range.



Get: Start(self: SelectionRange) -> DateTime



Set: Start(self: SelectionRange)=value

"""





class ICustomMarshaler:
 """ Provides custom wrappers for handling method calls. """
 def CleanUpManagedData(self,ManagedObj):
  """
  CleanUpManagedData(self: ICustomMarshaler,ManagedObj: object)

   Performs necessary cleanup of the managed data when it is no longer needed.

  

   ManagedObj: The managed object to be destroyed.
  """
  pass
 def CleanUpNativeData(self,pNativeData):
  """
  CleanUpNativeData(self: ICustomMarshaler,pNativeData: IntPtr)

   Performs necessary cleanup of the unmanaged data when it is no longer needed.

  

   pNativeData: A pointer to the unmanaged data to be destroyed.
  """
  pass
 def GetNativeDataSize(self):
  """
  GetNativeDataSize(self: ICustomMarshaler) -> int

  

   Returns the size of the native data to be marshaled.

   Returns: The size,in bytes,of the native data.
  """
  pass
 def MarshalManagedToNative(self,ManagedObj):
  """
  MarshalManagedToNative(self: ICustomMarshaler,ManagedObj: object) -> IntPtr

  

   Converts the managed data to unmanaged data.

  

   ManagedObj: The managed object to be converted.

   Returns: A pointer to the COM view of the managed object.
  """
  pass
 def MarshalNativeToManaged(self,pNativeData):
  """
  MarshalNativeToManaged(self: ICustomMarshaler,pNativeData: IntPtr) -> object

  

   Converts the unmanaged data to managed data.

  

   pNativeData: A pointer to the unmanaged data to be wrapped.

   Returns: An object that represents the managed view of the COM data.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class PropertyDescriptorCollection(object,ICollection,IEnumerable,IList,IDictionary):
 """
 Represents a collection of System.ComponentModel.PropertyDescriptor objects.

 

 PropertyDescriptorCollection(properties: Array[PropertyDescriptor])

 PropertyDescriptorCollection(properties: Array[PropertyDescriptor],readOnly: bool)
 """
 def Add(self,value):
  """
  Add(self: PropertyDescriptorCollection,value: PropertyDescriptor) -> int

  

   Adds the specified System.ComponentModel.PropertyDescriptor to the collection.

  

   value: The System.ComponentModel.PropertyDescriptor to add to the collection.

   Returns: The index of the System.ComponentModel.PropertyDescriptor that was added to the collection.
  """
  pass
 def Clear(self):
  """
  Clear(self: PropertyDescriptorCollection)

   Removes all System.ComponentModel.PropertyDescriptor objects from the collection.
  """
  pass
 def Contains(self,value):
  """
  Contains(self: PropertyDescriptorCollection,value: PropertyDescriptor) -> bool

  

   Returns whether the collection contains the given System.ComponentModel.PropertyDescriptor.

  

   value: The System.ComponentModel.PropertyDescriptor to find in the collection.

   Returns: true if the collection contains the given System.ComponentModel.PropertyDescriptor; otherwise,

    false.
  """
  pass
 def CopyTo(self,array,index):
  """
  CopyTo(self: PropertyDescriptorCollection,array: Array,index: int)

   Copies the entire collection to an array,starting at the specified index number.

  

   array: An array of System.ComponentModel.PropertyDescriptor objects to copy elements of the collection 

    to.

  

   index: The index of the array parameter at which copying begins.
  """
  pass
 def Find(self,name,ignoreCase):
  """
  Find(self: PropertyDescriptorCollection,name: str,ignoreCase: bool) -> PropertyDescriptor

  

   Returns the System.ComponentModel.PropertyDescriptor with the specified name,using a Boolean to 

    indicate whether to ignore case.

  

  

   name: The name of the System.ComponentModel.PropertyDescriptor to return from the collection.

   ignoreCase: true if you want to ignore the case of the property name; otherwise,false.

   Returns: A System.ComponentModel.PropertyDescriptor with the specified name,or null if the property does 

    not exist.
  """
  pass
 def GetEnumerator(self):
  """
  GetEnumerator(self: PropertyDescriptorCollection) -> IEnumerator

  

   Returns an enumerator for this class.

   Returns: An enumerator of type System.Collections.IEnumerator.
  """
  pass
 def IndexOf(self,value):
  """
  IndexOf(self: PropertyDescriptorCollection,value: PropertyDescriptor) -> int

  

   Returns the index of the given System.ComponentModel.PropertyDescriptor.

  

   value: The System.ComponentModel.PropertyDescriptor to return the index of.

   Returns: The index of the given System.ComponentModel.PropertyDescriptor.
  """
  pass
 def Insert(self,index,value):
  """
  Insert(self: PropertyDescriptorCollection,index: int,value: PropertyDescriptor)

   Adds the System.ComponentModel.PropertyDescriptor to the collection at the specified index 

    number.

  

  

   index: The index at which to add the value parameter to the collection.

   value: The System.ComponentModel.PropertyDescriptor to add to the collection.
  """
  pass
 def InternalSort(self,*args):
  """
  InternalSort(self: PropertyDescriptorCollection,sorter: IComparer)

   Sorts the members of this collection,using the specified System.Collections.IComparer.

  

   sorter: A comparer to use to sort the System.ComponentModel.PropertyDescriptor objects in this 

    collection.

  

  InternalSort(self: PropertyDescriptorCollection,names: Array[str])

   Sorts the members of this collection. The specified order is applied first,followed by the 

    default sort for this collection,which is usually alphabetical.

  

  

   names: An array of strings describing the order in which to sort the 

    System.ComponentModel.PropertyDescriptor objects in this collection.
  """
  pass
 def Remove(self,value):
  """
  Remove(self: PropertyDescriptorCollection,value: PropertyDescriptor)

   Removes the specified System.ComponentModel.PropertyDescriptor from the collection.

  

   value: The System.ComponentModel.PropertyDescriptor to remove from the collection.
  """
  pass
 def RemoveAt(self,index):
  """
  RemoveAt(self: PropertyDescriptorCollection,index: int)

   Removes the System.ComponentModel.PropertyDescriptor at the specified index from the collection.

  

   index: The index of the System.ComponentModel.PropertyDescriptor to remove from the collection.
  """
  pass
 def Sort(self,*__args):
  """
  Sort(self: PropertyDescriptorCollection,names: Array[str],comparer: IComparer) -> PropertyDescriptorCollection

  

   Sorts the members of this collection. The specified order is applied first,followed by the sort 

    using the specified System.Collections.IComparer.

  

  

   names: An array of strings describing the order in which to sort the 

    System.ComponentModel.PropertyDescriptor objects in this collection.

  

   comparer: A comparer to use to sort the System.ComponentModel.PropertyDescriptor objects in this 

    collection.

  

   Returns: A new System.ComponentModel.PropertyDescriptorCollection that contains the sorted 

    System.ComponentModel.PropertyDescriptor objects.

  

  Sort(self: PropertyDescriptorCollection,comparer: IComparer) -> PropertyDescriptorCollection

  

   Sorts the members of this collection,using the specified System.Collections.IComparer.

  

   comparer: A comparer to use to sort the System.ComponentModel.PropertyDescriptor objects in this 

    collection.

  

   Returns: A new System.ComponentModel.PropertyDescriptorCollection that contains the sorted 

    System.ComponentModel.PropertyDescriptor objects.

  

  Sort(self: PropertyDescriptorCollection) -> PropertyDescriptorCollection

  

   Sorts the members of this collection,using the default sort for this collection,which is 

    usually alphabetical.

  

   Returns: A new System.ComponentModel.PropertyDescriptorCollection that contains the sorted 

    System.ComponentModel.PropertyDescriptor objects.

  

  Sort(self: PropertyDescriptorCollection,names: Array[str]) -> PropertyDescriptorCollection

  

   Sorts the members of this collection. The specified order is applied first,followed by the 

    default sort for this collection,which is usually alphabetical.

  

  

   names: An array of strings describing the order in which to sort the 

    System.ComponentModel.PropertyDescriptor objects in this collection.

  

   Returns: A new System.ComponentModel.PropertyDescriptorCollection that contains the sorted 

    System.ComponentModel.PropertyDescriptor objects.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+y """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: IList,value: object) -> bool

  

   Determines whether the System.Collections.IList contains a specific value.

  

   value: The object to locate in the System.Collections.IList.

   Returns: true if the System.Object is found in the System.Collections.IList; otherwise,false.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 @staticmethod
 def __new__(self,properties,readOnly=None):
  """
  __new__(cls: type,properties: Array[PropertyDescriptor])

  __new__(cls: type,properties: Array[PropertyDescriptor],readOnly: bool)
  """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of property descriptors in the collection.



Get: Count(self: PropertyDescriptorCollection) -> int



"""


 Empty=None




class ImeContext(object):
 """ Contains static methods that interact directly with the IME API. """
 @staticmethod
 def Disable(handle):
  """
  Disable(handle: IntPtr)

   Disables the specified IME.

  

   handle: A pointer to the IME to disable.
  """
  pass
 @staticmethod
 def Enable(handle):
  """
  Enable(handle: IntPtr)

   Enables the specified IME.

  

   handle: A pointer to the IME to enable.
  """
  pass
 @staticmethod
 def GetImeMode(handle):
  """
  GetImeMode(handle: IntPtr) -> ImeMode

  

   Returns the System.Windows.Forms.ImeMode of the specified IME.

  

   handle: A pointer to the IME to query.

   Returns: The System.Windows.Forms.ImeMode of the specified IME.
  """
  pass
 @staticmethod
 def IsOpen(handle):
  """
  IsOpen(handle: IntPtr) -> bool

  

   Returns a value that indicates whether the specified IME is open.

  

   handle: A pointer to the IME to query.

   Returns: true if the specified IME is open; otherwise,false.
  """
  pass
 @staticmethod
 def SetImeStatus(imeMode,handle):
  """
  SetImeStatus(imeMode: ImeMode,handle: IntPtr)

   Sets the status of the specified IME.

  

   imeMode: The status to set.

   handle: A pointer to the IME to set status of.
  """
  pass
 @staticmethod
 def SetOpenStatus(open,handle):
  """
  SetOpenStatus(open: bool,handle: IntPtr)

   Opens or closes the IME context.

  

   open: true to open the IME or false to close it.

   handle: A pointer to the IME to open or close.
  """
  pass
 __all__=[
  'Disable',
  'Enable',
  'GetImeMode',
  'IsOpen',
  'SetImeStatus',
  'SetOpenStatus',
 ]




class PerformanceCounterType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the formula used to calculate the System.Diagnostics.PerformanceCounter.NextValue method for a System.Diagnostics.PerformanceCounter instance.

 

 enum PerformanceCounterType,values: AverageBase (1073939458),AverageCount64 (1073874176),AverageTimer32 (805438464),CounterDelta32 (4195328),CounterDelta64 (4195584),CounterMultiBase (1107494144),CounterMultiTimer (574686464),CounterMultiTimer100Ns (575735040),CounterMultiTimer100NsInverse (592512256),CounterMultiTimerInverse (591463680),CounterTimer (541132032),CounterTimerInverse (557909248),CountPerTimeInterval32 (4523008),CountPerTimeInterval64 (4523264),ElapsedTime (807666944),NumberOfItems32 (65536),NumberOfItems64 (65792),NumberOfItemsHEX32 (0),NumberOfItemsHEX64 (256),RateOfCountsPerSecond32 (272696320),RateOfCountsPerSecond64 (272696576),RawBase (1073939459),RawFraction (537003008),SampleBase (1073939457),SampleCounter (4260864),SampleFraction (549585920),Timer100Ns (542180608),Timer100NsInverse (558957824)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AverageBase=None
 AverageCount64=None
 AverageTimer32=None
 CounterDelta32=None
 CounterDelta64=None
 CounterMultiBase=None
 CounterMultiTimer=None
 CounterMultiTimer100Ns=None
 CounterMultiTimer100NsInverse=None
 CounterMultiTimerInverse=None
 CounterTimer=None
 CounterTimerInverse=None
 CountPerTimeInterval32=None
 CountPerTimeInterval64=None
 ElapsedTime=None
 NumberOfItems32=None
 NumberOfItems64=None
 NumberOfItemsHEX32=None
 NumberOfItemsHEX64=None
 RateOfCountsPerSecond32=None
 RateOfCountsPerSecond64=None
 RawBase=None
 RawFraction=None
 SampleBase=None
 SampleCounter=None
 SampleFraction=None
 Timer100Ns=None
 Timer100NsInverse=None
 value__=None




class PropertyTabChangedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.PropertyGrid.PropertyTabChanged event of a System.Windows.Forms.PropertyGrid.

 

 PropertyTabChangedEventArgs(oldTab: PropertyTab,newTab: PropertyTab)
 """
 @staticmethod
 def __new__(self,oldTab,newTab):
  """ __new__(cls: type,oldTab: PropertyTab,newTab: PropertyTab) """
  pass
 NewTab=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the new System.Windows.Forms.Design.PropertyTab selected.



Get: NewTab(self: PropertyTabChangedEventArgs) -> PropertyTab



"""

 OldTab=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the old System.Windows.Forms.Design.PropertyTab selected.



Get: OldTab(self: PropertyTabChangedEventArgs) -> PropertyTab



"""





class TypeValidationEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.MaskedTextBox.TypeValidationCompleted event.

 

 TypeValidationEventArgs(validatingType: Type,isValidInput: bool,returnValue: object,message: str)
 """
 @staticmethod
 def __new__(self,validatingType,isValidInput,returnValue,message):
  """ __new__(cls: type,validatingType: Type,isValidInput: bool,returnValue: object,message: str) """
  pass
 Cancel=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the event should be canceled.



Get: Cancel(self: TypeValidationEventArgs) -> bool



Set: Cancel(self: TypeValidationEventArgs)=value

"""

 IsValidInput=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the formatted input string was successfully converted to the validating type.



Get: IsValidInput(self: TypeValidationEventArgs) -> bool



"""

 Message=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a text message describing the conversion process.



Get: Message(self: TypeValidationEventArgs) -> str



"""

 ReturnValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the object that results from the conversion of the formatted input string.



Get: ReturnValue(self: TypeValidationEventArgs) -> object



"""

 ValidatingType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the type that the formatted input string is being validated against.



Get: ValidatingType(self: TypeValidationEventArgs) -> Type



"""





class TreeNodeMouseHoverEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.TreeView.NodeMouseHover event.

 

 TreeNodeMouseHoverEventArgs(node: TreeNode)
 """
 @staticmethod
 def __new__(self,node):
  """ __new__(cls: type,node: TreeNode) """
  pass
 Node=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the node the mouse pointer is currently resting on.



Get: Node(self: TreeNodeMouseHoverEventArgs) -> TreeNode



"""





"""Revit transactions facility."""
from pyrevit import HOST_APP, DOCS, DB
from pyrevit import coreutils
from pyrevit.coreutils.logger import get_logger
from pyrevit.revit.db import failure


__all__ = ('carryout',
           'Transaction', 'DryTransaction', 'TransactionGroup')


#pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)


DEFAULT_TRANSACTION_NAME = 'pyRevit Transaction'


class Transaction():
    """Adds a context manager around Revit Transaction object.

    Runs `Transaction.Start()` and `Transaction.Commit()`
    before and after the context.
    Automatically rolls back if exception is raised.

    ```python
        with Transaction('Move Wall'):
            wall.DoSomething()

        with Transaction('Move Wall', doc, clear_after_rollback=False, show_error_dialog=False, swallow_errors=False, log_errors=True, nested=False) as action:
            wall.DoSomething()
            assert action.status == ActionStatus.Started  # True
            assert action.status == ActionStatus.Committed    # True
        ```
    """
    def __init__(self, name=None,
                 doc=None,
                 clear_after_rollback=False,
                 show_error_dialog=False,
                 swallow_errors=False,
                 log_errors=True,
                 nested=False):
        doc = doc or DOCS.doc
        # create nested transaction if one is already open
        if doc.IsModifiable or nested:
            self._rvtxn = \
                DB.SubTransaction(doc)
        else:
            self._rvtxn = \
                DB.Transaction(doc, name if name else DEFAULT_TRANSACTION_NAME)
            self._fhndlr_ops = self._rvtxn.GetFailureHandlingOptions()
            self._fhndlr_ops = \
                self._fhndlr_ops.SetClearAfterRollback(clear_after_rollback)
            self._fhndlr_ops = \
                self._fhndlr_ops.SetForcedModalHandling(show_error_dialog)
            if swallow_errors:
                self._fhndlr_ops = \
                    self._fhndlr_ops.SetFailuresPreprocessor(
                        failure.FailureSwallower()
                        )
            self._rvtxn.SetFailureHandlingOptions(self._fhndlr_ops)
        self._logerror = log_errors

    def __enter__(self):
        self._rvtxn.Start()
        return self

    def __exit__(self, exception, exception_value, traceback):
        if exception:
            self._rvtxn.RollBack()
            if self._logerror:
                mlogger.error('Error in Transaction Context. '
                              'Rolling back changes. | %s:%s',
                              exception, exception_value)
        else:
            try:
                self._rvtxn.Commit()
            except Exception as errmsg:
                self._rvtxn.RollBack()
                if self._logerror:
                    mlogger.error('Error in Transaction Commit. '
                                  'Rolling back changes. | %s', errmsg)
        self._rvtxn.Dispose()

    @property
    def name(self):
        if hasattr(self._rvtxn, 'GetName'):
            return self._rvtxn.GetName()

    @name.setter
    def name(self, new_name):
        if hasattr(self._rvtxn, 'SetName'):
            return self._rvtxn.SetName(new_name)

    @property
    def status(self):
        return self._rvtxn.GetStatus()

    def has_started(self):
        return self._rvtxn.HasStarted()

    def has_ended(self):
        return self._rvtxn.HasEnded()


class DryTransaction(Transaction):
    """Wrapper to a transaction that doesn't commit anything (dry-run)."""
    def __exit__(self, exception, exception_value, traceback):
        self._rvtxn.RollBack()
        self._rvtxn.Dispose()


class TransactionGroup():
    """Transactions group with context manager."""
    def __init__(self, name=None, doc=None, assimilate=True, log_errors=True):
        self._rvtxn_grp = \
            DB.TransactionGroup(doc or DOCS.doc,
                                name if name else DEFAULT_TRANSACTION_NAME)
        self.assimilate = assimilate
        self._logerror = log_errors

    def __enter__(self):
        self._rvtxn_grp.Start()
        return self

    def __exit__(self, exception, exception_value, traceback):
        if exception:
            self._rvtxn_grp.RollBack()
            if self._logerror:
                mlogger.error('Error in TransactionGroup Context. '
                              'Rolling back changes. | %s:%s',
                              exception, exception_value)
        else:
            try:
                if self.assimilate:
                    self._rvtxn_grp.Assimilate()
                else:
                    self._rvtxn_grp.Commit()
            except Exception as errmsg:
                self._rvtxn_grp.RollBack()
                if self._logerror:
                    mlogger.error('Error in TransactionGroup Commit. '
                                  'Rolling back changes. | %s', errmsg)
        self._rvtxn_grp.Dispose()

    @property
    def name(self):
        return self._rvtxn_grp.GetName()

    @name.setter
    def name(self, new_name):
        return self._rvtxn_grp.SetName(new_name)

    @property
    def status(self):
        return self._rvtxn_grp.GetStatus()

    def has_started(self):
        return self._rvtxn_grp.HasStarted()

    def has_ended(self):
        return self._rvtxn_grp.HasEnded()


def carryout(name, doc=None):
    """Transaction Decorator.

    Decorate any function with ``@doc.carryout('Txn name')``
    and the funciton will run within an Transaction context.

    Args:
        name (str): Name of the Transaction
        doc (Document): Revit document

    ```python
    @doc.carryout('Do Something')
    def set_some_parameter(wall, value):
        wall.parameters['Comments'].value = value
    
    
    set_some_parameter(wall, value)
    ```
    """
    from functools import wraps

    def wrap(f):
        @wraps(f)
        def wrapped_f(*args, **kwargs):
            with Transaction(name, doc=doc):
                return_value = f(*args, **kwargs)
            return return_value
        return wrapped_f
    return wrap



class DataGridColumnEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Controls.DataGrid.ColumnDisplayIndexChanged and System.Windows.Controls.DataGrid.ColumnReordered events.

 

 DataGridColumnEventArgs(column: DataGridColumn)
 """
 @staticmethod
 def __new__(self,column):
  """ __new__(cls: type,column: DataGridColumn) """
  pass
 Column=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the column related to the event.



Get: Column(self: DataGridColumnEventArgs) -> DataGridColumn



"""





class DataGridColumn(DependencyObject):
 """ Represents a System.Windows.Controls.DataGrid column. """
 def CancelCellEdit(self,*args):
  """
  CancelCellEdit(self: DataGridColumn,editingElement: FrameworkElement,uneditedValue: object)

   Causes the cell being edited to revert to the original,unedited value.

  

   editingElement: The element that the column displays for a cell in editing mode.

   uneditedValue: The original,unedited value in the cell being edited.
  """
  pass
 def CommitCellEdit(self,*args):
  """
  CommitCellEdit(self: DataGridColumn,editingElement: FrameworkElement) -> bool

  

   Performs any required validation before exiting cell editing mode.

  

   editingElement: The element that the column displays for a cell in editing mode.

   Returns: true if no validation errors are found; otherwise,false.
  """
  pass
 def GenerateEditingElement(self,*args):
  """
  GenerateEditingElement(self: DataGridColumn,cell: DataGridCell,dataItem: object) -> FrameworkElement

  

   When overridden in a derived class,gets an editing element that is bound to the 

    System.Windows.Controls.DataGridBoundColumn.Binding property value of the column.

  

  

   cell: The cell that will contain the generated element.

   dataItem: The data item that is represented by the row that contains the intended cell.

   Returns: A new editing element that is bound to the System.Windows.Controls.DataGridBoundColumn.Binding 

    property value of the column.
  """
  pass
 def GenerateElement(self,*args):
  """
  GenerateElement(self: DataGridColumn,cell: DataGridCell,dataItem: object) -> FrameworkElement

  

   When overridden in a derived class,gets a read-only element that is bound to the 

    System.Windows.Controls.DataGridBoundColumn.Binding property value of the column.

  

  

   cell: The cell that will contain the generated element.

   dataItem: The data item that is represented by the row that contains the intended cell.

   Returns: A new read-only element that is bound to the System.Windows.Controls.DataGridBoundColumn.Binding 

    property value of the column.
  """
  pass
 def GetCellContent(self,*__args):
  """
  GetCellContent(self: DataGridColumn,dataGridRow: DataGridRow) -> FrameworkElement

  

   Retrieves the System.Windows.Controls.ContentControl.Content property value for the cell at the 

    intersection of this column and the specified row.

  

  

   dataGridRow: The row that contains the intended cell.

   Returns: The cell content; or null,if the cell is not found.

  GetCellContent(self: DataGridColumn,dataItem: object) -> FrameworkElement

  

   Gets the System.Windows.Controls.ContentControl.Content property value for the cell at the 

    intersection of this column and the row that represents the specified data item.

  

  

   dataItem: The data item that is represented by the row that contains the intended cell.

   Returns: The cell content; or null,if the cell is not found.
  """
  pass
 def NotifyPropertyChanged(self,*args):
  """
  NotifyPropertyChanged(self: DataGridColumn,propertyName: str)

   Notifies the System.Windows.Controls.DataGrid that contains this column that a column property 

    has changed.

  

  

   propertyName: The name of the column property that changed.
  """
  pass
 def OnCoerceIsReadOnly(self,*args):
  """
  OnCoerceIsReadOnly(self: DataGridColumn,baseValue: bool) -> bool

  

   Determines the value of the System.Windows.Controls.DataGridColumn.IsReadOnly property based on 

    the property rules of the System.Windows.Controls.DataGrid that contains this column.

  

  

   baseValue: The value that was passed to the delegate.

   Returns: true if cells in the column cannot be edited based on rules from the 

    System.Windows.Controls.DataGrid; otherwise,false.
  """
  pass
 def OnCopyingCellClipboardContent(self,item):
  """
  OnCopyingCellClipboardContent(self: DataGridColumn,item: object) -> object

  

   Raises the System.Windows.Controls.DataGridColumn.CopyingCellClipboardContent event.

  

   item: The data context for the selected element.

   Returns: An object that represents the content of the cell.
  """
  pass
 def OnPastingCellClipboardContent(self,item,cellContent):
  """
  OnPastingCellClipboardContent(self: DataGridColumn,item: object,cellContent: object)

   Raises the System.Windows.Controls.DataGridColumn.PastingCellClipboardContent event.

  

   item: The data context for the selected element.

   cellContent: The content to paste into the cell.
  """
  pass
 def PrepareCellForEdit(self,*args):
  """
  PrepareCellForEdit(self: DataGridColumn,editingElement: FrameworkElement,editingEventArgs: RoutedEventArgs) -> object

  

   When overridden in a derived class,sets cell content as needed for editing.

  

   editingElement: The element that the column displays for a cell in editing mode.

   editingEventArgs: Information about the user gesture that is causing a cell to enter editing mode.

   Returns: When returned by a derived class,the unedited cell value. This implementation returns null in 

    all cases.
  """
  pass
 def RefreshCellContent(self,*args):
  """
  RefreshCellContent(self: DataGridColumn,element: FrameworkElement,propertyName: str)

   When overridden in a derived class,updates the contents of a cell in the column in response to 

    a column property value that changed.

  

  

   element: The cell to update.

   propertyName: The name of the column property that changed.
  """
  pass
 ActualWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the current width of the column,in device-independent units (1/96th inch per unit).



Get: ActualWidth(self: DataGridColumn) -> float



"""

 CanUserReorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the user can change the column display position by dragging the column header.



Get: CanUserReorder(self: DataGridColumn) -> bool



Set: CanUserReorder(self: DataGridColumn)=value

"""

 CanUserResize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the user can adjust the column width by using the mouse.



Get: CanUserResize(self: DataGridColumn) -> bool



Set: CanUserResize(self: DataGridColumn)=value

"""

 CanUserSort=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the user can sort the column by clicking the column header.



Get: CanUserSort(self: DataGridColumn) -> bool



Set: CanUserSort(self: DataGridColumn)=value

"""

 CellStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the style that is used to render cells in the column.



Get: CellStyle(self: DataGridColumn) -> Style



Set: CellStyle(self: DataGridColumn)=value

"""

 ClipboardContentBinding=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the binding object to use when getting or setting cell content for the clipboard.



Get: ClipboardContentBinding(self: DataGridColumn) -> BindingBase



Set: ClipboardContentBinding(self: DataGridColumn)=value

"""

 DataGridOwner=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Controls.DataGrid control that contains this column.



"""

 DisplayIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the display position of the column relative to the other columns in the System.Windows.Controls.DataGrid.



Get: DisplayIndex(self: DataGridColumn) -> int



Set: DisplayIndex(self: DataGridColumn)=value

"""

 DragIndicatorStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the style object to apply to the column header during a drag operation.



Get: DragIndicatorStyle(self: DataGridColumn) -> Style



Set: DragIndicatorStyle(self: DataGridColumn)=value

"""

 Header=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the content of the column header.



Get: Header(self: DataGridColumn) -> object



Set: Header(self: DataGridColumn)=value

"""

 HeaderStringFormat=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the format pattern to apply to the content of the column header.



Get: HeaderStringFormat(self: DataGridColumn) -> str



Set: HeaderStringFormat(self: DataGridColumn)=value

"""

 HeaderStyle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the style that is used when rendering the column header.



Get: HeaderStyle(self: DataGridColumn) -> Style



Set: HeaderStyle(self: DataGridColumn)=value

"""

 HeaderTemplate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the template that defines the visual representation of the column header.



Get: HeaderTemplate(self: DataGridColumn) -> DataTemplate



Set: HeaderTemplate(self: DataGridColumn)=value

"""

 HeaderTemplateSelector=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the object that selects which template to use for the column header.



Get: HeaderTemplateSelector(self: DataGridColumn) -> DataTemplateSelector



Set: HeaderTemplateSelector(self: DataGridColumn)=value

"""

 IsAutoGenerated=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the column is auto-generated.



Get: IsAutoGenerated(self: DataGridColumn) -> bool



"""

 IsFrozen=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the column is prevented from scrolling horizontally.



Get: IsFrozen(self: DataGridColumn) -> bool



"""

 IsReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether cells in the column can be edited.



Get: IsReadOnly(self: DataGridColumn) -> bool



Set: IsReadOnly(self: DataGridColumn)=value

"""

 MaxWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the maximum width constraint of the column.



Get: MaxWidth(self: DataGridColumn) -> float



Set: MaxWidth(self: DataGridColumn)=value

"""

 MinWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the minimum width constraint of the column.



Get: MinWidth(self: DataGridColumn) -> float



Set: MinWidth(self: DataGridColumn)=value

"""

 SortDirection=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the sort direction (ascending or descending) of the column.



Get: SortDirection(self: DataGridColumn) -> Nullable[ListSortDirection]



Set: SortDirection(self: DataGridColumn)=value

"""

 SortMemberPath=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a property name,or a period-delimited hierarchy of property names,that indicates the member to sort by.



Get: SortMemberPath(self: DataGridColumn) -> str



Set: SortMemberPath(self: DataGridColumn)=value

"""

 Visibility=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the visibility of the column.



Get: Visibility(self: DataGridColumn) -> Visibility



Set: Visibility(self: DataGridColumn)=value

"""

 Width=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the column width or automatic sizing mode.



Get: Width(self: DataGridColumn) -> DataGridLength



Set: Width(self: DataGridColumn)=value

"""


 ActualWidthProperty=None
 CanUserReorderProperty=None
 CanUserResizeProperty=None
 CanUserSortProperty=None
 CellStyleProperty=None
 CopyingCellClipboardContent=None
 DisplayIndexProperty=None
 DragIndicatorStyleProperty=None
 HeaderProperty=None
 HeaderStringFormatProperty=None
 HeaderStyleProperty=None
 HeaderTemplateProperty=None
 HeaderTemplateSelectorProperty=None
 IsAutoGeneratedProperty=None
 IsFrozenProperty=None
 IsReadOnlyProperty=None
 MaxWidthProperty=None
 MinWidthProperty=None
 PastingCellClipboardContent=None
 SortDirectionProperty=None
 SortMemberPathProperty=None
 VisibilityProperty=None
 WidthProperty=None




"""Base module to handle extension ASCII caching."""
import json
import codecs

from pyrevit import PyRevitException
from pyrevit.coreutils import appdata
from pyrevit.coreutils import get_all_subclasses
from pyrevit.coreutils.logger import get_logger
from pyrevit.extensions import components as comps
from pyrevit.extensions import genericcomps as gencomps

#pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)


def _get_cache_file(cached_ext):
    return appdata.get_data_file(file_id='cache_{}'.format(cached_ext.name),
                                 file_ext='json')


def _make_cache_from_cmp(obj):
    return json.dumps(obj,
                      default=lambda o: o.get_cache_data(),
                      sort_keys=True,
                      indent=4,
                      ensure_ascii=False)


def _make_layoutitems_from_cache(parent_cmp, cached_layoutitems):
    if hasattr(parent_cmp, gencomps.LAYOUT_ITEM_KEY):
        litems = []
        for litem_cache in cached_layoutitems:
            # grab the layout directive cache and create component
            ldir = None
            ldir_cache = litem_cache.get(gencomps.LAYOUT_DIR_KEY, {})
            if ldir_cache:
                ldir = gencomps.LayoutDirective()
                ldir.load_cache_data(ldir_cache)

            litem = gencomps.LayoutItem()
            litem.load_cache_data(litem_cache)
            # set the layout directive
            setattr(litem, gencomps.LAYOUT_DIR_KEY, ldir)
            litems.append(litem)
        setattr(parent_cmp, gencomps.LAYOUT_ITEM_KEY, litems)


def _make_sub_cmp_from_cache(parent_cmp, cached_sub_cmps):
    mlogger.debug('Processing cache for: %s', parent_cmp)
    # get allowed classes under this component
    allowed_sub_cmps = get_all_subclasses(parent_cmp.allowed_sub_cmps)
    mlogger.debug('Allowed sub components are: %s', allowed_sub_cmps)
    # iterate through list of cached sub components
    for cached_cmp in cached_sub_cmps:  # type: dict
        for sub_class in allowed_sub_cmps:
            if sub_class.type_id == cached_cmp[gencomps.TYPE_ID_KEY]:
                mlogger.debug('Creating sub component from cache: %s, %s',
                              cached_cmp[gencomps.NAME_KEY], sub_class)

                # cached_cmp might contain gencomps.SUB_CMP_KEY. This needs to be
                # removed since this function will make all the children
                # recursively. So if this component has gencomps.SUB_CMP_KEY means
                # it has sub components:
                sub_cmp_cache = None
                if gencomps.SUB_CMP_KEY in cached_cmp.keys():
                    # drop subcomponents dict from cached_cmp since we
                    # don't want the loaded_cmp to include this
                    sub_cmp_cache = cached_cmp.pop(gencomps.SUB_CMP_KEY)

                # grab the layout items
                # layoutitems_cache is list[dict]
                layoutitems_cache = None
                if gencomps.LAYOUT_ITEM_KEY in cached_cmp.keys():
                    # drop layoutitems dict from cached_cmp since we
                    # don't want the loaded_cmp to include this
                    layoutitems_cache = cached_cmp.pop(gencomps.LAYOUT_ITEM_KEY)

                # create sub component from cleaned cached_cmp
                loaded_cmp = sub_class()
                loaded_cmp.load_cache_data(cached_cmp)

                # now process sub components for loaded_cmp if any
                if sub_cmp_cache:
                    _make_sub_cmp_from_cache(loaded_cmp, sub_cmp_cache)

                # apply layout items
                if layoutitems_cache:
                    _make_layoutitems_from_cache(loaded_cmp, layoutitems_cache)

                parent_cmp.add_component(loaded_cmp)


def _read_cache_for(cached_ext):
    try:
        mlogger.debug('Reading cache for: %s', cached_ext)
        cache_file = _get_cache_file(cached_ext)
        mlogger.debug('Cache file is: %s', cache_file)
        with codecs.open(_get_cache_file(cached_ext), 'r', 'utf-8') \
                as cache_file:
            cached_tab_dict = json.load(cache_file)
        return cached_tab_dict
    except Exception as err:
        raise PyRevitException('Error reading cache for: {} | {}'
                               .format(cached_ext, err))


def _write_cache_for(parsed_ext):
    try:
        mlogger.debug('Writing cache for: %s', parsed_ext)
        cache_file = _get_cache_file(parsed_ext)
        mlogger.debug('Cache file is: %s', cache_file)
        with codecs.open(cache_file, 'w', 'utf-8') as cache_file:
            cache_file.write(_make_cache_from_cmp(parsed_ext))
    except Exception as err:
        mlogger.debug('Error writing cache...')
        raise PyRevitException('Error writing cache for: {} | {}'
                               .format(parsed_ext, err))


def update_cache(parsed_ext):
    mlogger.debug('Updating cache for tab: %s ...', parsed_ext.name)
    _write_cache_for(parsed_ext)
    mlogger.debug('Cache updated for tab: %s', parsed_ext.name)


def get_cached_extension(installed_ext):
    cached_ext_dict = _read_cache_for(installed_ext)
    # try:
    mlogger.debug('Constructing components from cache for: %s',
                    installed_ext)
    # get cached sub component dictionary and call recursive maker function
    _make_sub_cmp_from_cache(installed_ext,
                                cached_ext_dict.pop(gencomps.SUB_CMP_KEY))
    mlogger.debug('Load successful...')
    # except Exception as err:
    #     mlogger.debug('Error reading cache...')
    #     raise PyRevitException('Error creating ext from cache for: {} | {}'
    #                            .format(installed_ext.name, err))

    return installed_ext


def is_cache_valid(extension):
    try:
        cached_ext_dict = _read_cache_for(extension)  # type: dict
        mlogger.debug('Extension cache directory is: %s for: %s',
                      extension.directory, extension)
        cache_dir_valid = cached_ext_dict[gencomps.EXT_DIR_KEY] == extension.directory

        mlogger.debug('Extension cache version is: %s for: %s',
                      extension.pyrvt_version, extension)
        cache_version_valid = \
            cached_ext_dict[comps.EXT_HASH_VERSION_KEY] == extension.pyrvt_version

        mlogger.debug('Extension hash value is: %s for:%s',
                      extension.dir_hash_value, extension)
        cache_hash_valid = \
            cached_ext_dict[comps.EXT_HASH_VALUE_KEY] == extension.dir_hash_value

        cache_valid = \
            cache_dir_valid and cache_version_valid and cache_hash_valid

        # cache is valid if both version and hash value match
        return cache_valid

    except PyRevitException as err:
        mlogger.debug(err)
        return False

    except Exception as err:
        mlogger.debug('Error determining cache validity: %s | %s',
                      extension, err)



class TraceSource(object):
 """
 Provides a set of methods and properties that enable applications to trace the execution of code and associate trace messages with their source.

 

 TraceSource(name: str)

 TraceSource(name: str,defaultLevel: SourceLevels)
 """
 def Close(self):
  """
  Close(self: TraceSource)

   Closes all the trace listeners in the trace listener collection.
  """
  pass
 def Flush(self):
  """
  Flush(self: TraceSource)

   Flushes all the trace listeners in the trace listener collection.
  """
  pass
 def GetSupportedAttributes(self,*args):
  """
  GetSupportedAttributes(self: TraceSource) -> Array[str]

  

   Gets the custom attributes supported by the trace source.

   Returns: A string array naming the custom attributes supported by the trace source,or null if there are 

    no custom attributes.
  """
  pass
 def TraceData(self,eventType,id,data):
  """
  TraceData(self: TraceSource,eventType: TraceEventType,id: int,*data: Array[object])

   Writes trace data to the trace listeners in the System.Diagnostics.TraceSource.Listeners 

    collection using the specified event type,event identifier,and trace data array.

  

  

   eventType: One of the enumeration values that specifies the event type of the trace data.

   id: A numeric identifier for the event.

   data: An object array containing the trace data.

  TraceData(self: TraceSource,eventType: TraceEventType,id: int,data: object)

   Writes trace data to the trace listeners in the System.Diagnostics.TraceSource.Listeners 

    collection using the specified event type,event identifier,and trace data.

  

  

   eventType: One of the enumeration values that specifies the event type of the trace data.

   id: A numeric identifier for the event.

   data: The trace data.
  """
  pass
 def TraceEvent(self,eventType,id,*__args):
  """
  TraceEvent(self: TraceSource,eventType: TraceEventType,id: int,format: str,*args: Array[object])

   Writes a trace event to the trace listeners in the System.Diagnostics.TraceSource.Listeners 

    collection using the specified event type,event identifier,and argument array and format.

  

  

   eventType: One of the enumeration values that specifies the event type of the trace data.

   id: A numeric identifier for the event.

   format: A composite format string (see Remarks) that contains text intermixed with zero or more format 

    items,which correspond to objects in the args array.

  

   args: An object array containing zero or more objects to format.

  TraceEvent(self: TraceSource,eventType: TraceEventType,id: int,message: str)

   Writes a trace event message to the trace listeners in the 

    System.Diagnostics.TraceSource.Listeners collection using the specified event type,event 

    identifier,and message.

  

  

   eventType: One of the enumeration values that specifies the event type of the trace data.

   id: A numeric identifier for the event.

   message: The trace message to write.

  TraceEvent(self: TraceSource,eventType: TraceEventType,id: int)

   Writes a trace event message to the trace listeners in the 

    System.Diagnostics.TraceSource.Listeners collection using the specified event type and event 

    identifier.

  

  

   eventType: One of the enumeration values that specifies the event type of the trace data.

   id: A numeric identifier for the event.
  """
  pass
 def TraceInformation(self,*__args):
  """
  TraceInformation(self: TraceSource,format: str,*args: Array[object])

   Writes an informational message to the trace listeners in the 

    System.Diagnostics.TraceSource.Listeners collection using the specified object array and 

    formatting information.

  

  

   format: A composite format string (see Remarks) that contains text intermixed with zero or more format 

    items,which correspond to objects in the args array.

  

   args: An array containing zero or more objects to format.

  TraceInformation(self: TraceSource,message: str)

   Writes an informational message to the trace listeners in the 

    System.Diagnostics.TraceSource.Listeners collection using the specified message.

  

  

   message: The informative message to write.
  """
  pass
 def TraceTransfer(self,id,message,relatedActivityId):
  """
  TraceTransfer(self: TraceSource,id: int,message: str,relatedActivityId: Guid)

   Writes a trace transfer message to the trace listeners in the 

    System.Diagnostics.TraceSource.Listeners collection using the specified numeric identifier,

    message,and related activity identifier.

  

  

   id: A numeric identifier for the event.

   message: The trace message to write.

   relatedActivityId: A structure that identifies the related activity.
  """
  pass
 @staticmethod
 def __new__(self,name,defaultLevel=None):
  """
  __new__(cls: type,name: str)

  __new__(cls: type,name: str,defaultLevel: SourceLevels)
  """
  pass
 Attributes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the custom switch attributes defined in the application configuration file.



Get: Attributes(self: TraceSource) -> StringDictionary



"""

 Listeners=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of trace listeners for the trace source.



Get: Listeners(self: TraceSource) -> TraceListenerCollection



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the trace source.



Get: Name(self: TraceSource) -> str



"""

 Switch=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the source switch value.



Get: Switch(self: TraceSource) -> SourceSwitch



Set: Switch(self: TraceSource)=value

"""





class DataGridViewColumnStateChangedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.DataGridView.ColumnStateChanged event.

 

 DataGridViewColumnStateChangedEventArgs(dataGridViewColumn: DataGridViewColumn,stateChanged: DataGridViewElementStates)
 """
 @staticmethod
 def __new__(self,dataGridViewColumn,stateChanged):
  """ __new__(cls: type,dataGridViewColumn: DataGridViewColumn,stateChanged: DataGridViewElementStates) """
  pass
 Column=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the column whose state changed.



Get: Column(self: DataGridViewColumnStateChangedEventArgs) -> DataGridViewColumn



"""

 StateChanged=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the new column state.



Get: StateChanged(self: DataGridViewColumnStateChangedEventArgs) -> DataGridViewElementStates



"""





class UriTypeConverter(TypeConverter):
 """
 Converts a System.String type to a System.Uri type,and vice versa.

 

 UriTypeConverter()
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: UriTypeConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Returns whether this converter can convert an object of the given type to the type of this 

    converter.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext  that provides a format context.

   sourceType: A System.Type that represents the type that you want to convert from.

   Returns: true if sourceType is a System.String type or a System.Uri type can be assigned from sourceType; 

    otherwise,false.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: UriTypeConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Returns whether this converter can convert the object to the specified type,using the specified 

    context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext  that provides a format context.

   destinationType: A System.Type that represents the type that you want to convert to.

   Returns: true if destinationType is of type 

    System.ComponentModel.Design.Serialization.InstanceDescriptor,System.String,or System.Uri; 

    otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: UriTypeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the given object to the type of this converter,using the specified context and culture 

    information.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext  that provides a format context.

   culture: The System.Globalization.CultureInfo to use as the current culture.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: UriTypeConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts a given value object to the specified type,using the specified context and culture 

    information.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext  that provides a format context.

   culture: A System.Globalization.CultureInfo. If null is passed,the current culture is assumed.

   value: The System.Object to convert.

   destinationType: The System.Type to convert the value parameter to.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def IsValid(self,*__args):
  """
  IsValid(self: UriTypeConverter,context: ITypeDescriptorContext,value: object) -> bool

  

   Returns whether the given value object is a System.Uri or a System.Uri can be created from it.

  

   context: An System.ComponentModel.ITypeDescriptorContext  that provides a format context.

   value: The System.Object to test for validity.

   Returns: true if value is a System.Uri or a System.String from which a System.Uri can be created; 

    otherwise,false.
  """
  pass



class NotFiniteNumberException(ArithmeticException,ISerializable,_Exception):
 """
 The exception that is thrown when a floating-point value is positive infinity,negative infinity,or Not-a-Number (NaN).

 

 NotFiniteNumberException()

 NotFiniteNumberException(offendingNumber: float)

 NotFiniteNumberException(message: str)

 NotFiniteNumberException(message: str,offendingNumber: float)

 NotFiniteNumberException(message: str,innerException: Exception)

 NotFiniteNumberException(message: str,offendingNumber: float,innerException: Exception)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: NotFiniteNumberException,info: SerializationInfo,context: StreamingContext)

   Sets the System.Runtime.Serialization.SerializationInfo object with the invalid number and 

    additional exception information.

  

  

   info: The object that holds the serialized object data.

   context: The contextual information about the source or destination.
  """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,offendingNumber: float)

  __new__(cls: type,message: str)

  __new__(cls: type,message: str,offendingNumber: float)

  __new__(cls: type,message: str,innerException: Exception)

  __new__(cls: type,message: str,offendingNumber: float,innerException: Exception)

  __new__(cls: type,info: SerializationInfo,context: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 OffendingNumber=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the invalid number that is a positive infinity,a negative infinity,or Not-a-Number (NaN).



Get: OffendingNumber(self: NotFiniteNumberException) -> float



"""





class DataGridViewRowStateChangedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.DataGridView.RowStateChanged event of a System.Windows.Forms.DataGridView.

 

 DataGridViewRowStateChangedEventArgs(dataGridViewRow: DataGridViewRow,stateChanged: DataGridViewElementStates)
 """
 @staticmethod
 def __new__(self,dataGridViewRow,stateChanged):
  """ __new__(cls: type,dataGridViewRow: DataGridViewRow,stateChanged: DataGridViewElementStates) """
  pass
 Row=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.DataGridViewRow that has a changed state.



Get: Row(self: DataGridViewRowStateChangedEventArgs) -> DataGridViewRow



"""

 StateChanged=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the state that has changed on the row.



Get: StateChanged(self: DataGridViewRowStateChangedEventArgs) -> DataGridViewElementStates



"""





class ControlPaint(object):
 """ Provides methods used to paint common Windows controls and their elements. This class cannot be inherited. """
 @staticmethod
 def CreateHBitmap16Bit(bitmap,background):
  """
  CreateHBitmap16Bit(bitmap: Bitmap,background: Color) -> IntPtr

  

   Creates a 16-bit color bitmap.

  

   bitmap: The System.Drawing.Bitmap to create.

   background: The System.Drawing.Color of the background.

   Returns: An System.IntPtr representing the handle to the bitmap.
  """
  pass
 @staticmethod
 def CreateHBitmapColorMask(bitmap,monochromeMask):
  """
  CreateHBitmapColorMask(bitmap: Bitmap,monochromeMask: IntPtr) -> IntPtr

  

   Creates a Win32 HBITMAP out of the image.

  

   bitmap: The System.Drawing.Bitmap to create.

   monochromeMask: A pointer to the monochrome mask.

   Returns: An System.IntPtr representing the handle to the bitmap.
  """
  pass
 @staticmethod
 def CreateHBitmapTransparencyMask(bitmap):
  """
  CreateHBitmapTransparencyMask(bitmap: Bitmap) -> IntPtr

  

   Creates a color mask for the specified bitmap that indicates which color should be displayed as 

    transparent.

  

  

   bitmap: The System.Drawing.Bitmap to create the transparency mask for.

   Returns: The handle to the System.Drawing.Bitmap mask.
  """
  pass
 @staticmethod
 def Dark(baseColor,percOfDarkDark=None):
  """
  Dark(baseColor: Color) -> Color

  

   Creates a new dark color object for the control from the specified color.

  

   baseColor: The System.Drawing.Color to be darkened.

   Returns: A System.Drawing.Color that represents the dark color on the control.

  Dark(baseColor: Color,percOfDarkDark: Single) -> Color

  

   Creates a new dark color object for the control from the specified color and darkens it by the 

    specified percentage.

  

  

   baseColor: The System.Drawing.Color to be darkened.

   percOfDarkDark: The percentage to darken the specified System.Drawing.Color.

   Returns: A System.Drawing.Color that represent the dark color on the control.
  """
  pass
 @staticmethod
 def DarkDark(baseColor):
  """
  DarkDark(baseColor: Color) -> Color

  

   Creates a new dark color object for the control from the specified color.

  

   baseColor: The System.Drawing.Color to be darkened.

   Returns: A System.Drawing.Color that represents the dark color on the control.
  """
  pass
 @staticmethod
 def DrawBorder(graphics,bounds,*__args):
  """
  DrawBorder(graphics: Graphics,bounds: Rectangle,leftColor: Color,leftWidth: int,leftStyle: ButtonBorderStyle,topColor: Color,topWidth: int,topStyle: ButtonBorderStyle,rightColor: Color,rightWidth: int,rightStyle: ButtonBorderStyle,bottomColor: Color,bottomWidth: int,bottomStyle: ButtonBorderStyle)

   Draws a border on a button-style control with the specified styles,colors,and border widths; 

    on the specified graphics surface; and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   bounds: The System.Drawing.Rectangle that represents the dimensions of the border.

   leftColor: The System.Drawing.Color of the left of the border.

   leftWidth: The width of the left border.

   leftStyle: One of the System.Windows.Forms.ButtonBorderStyle values that specifies the style of the left 

    border.

  

   topColor: The System.Drawing.Color of the top of the border.

   topWidth: The width of the top border.

   topStyle: One of the System.Windows.Forms.ButtonBorderStyle values that specifies the style of the top 

    border.

  

   rightColor: The System.Drawing.Color of the right of the border.

   rightWidth: The width of the right border.

   rightStyle: One of the System.Windows.Forms.ButtonBorderStyle values that specifies the style of the right 

    border.

  

   bottomColor: The System.Drawing.Color of the bottom of the border.

   bottomWidth: The width of the bottom border.

   bottomStyle: One of the System.Windows.Forms.ButtonBorderStyle values that specifies the style of the bottom 

    border.

  

  DrawBorder(graphics: Graphics,bounds: Rectangle,color: Color,style: ButtonBorderStyle)

   Draws a border with the specified style and color,on the specified graphics surface,and within 

    the specified bounds on a button-style control.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   bounds: The System.Drawing.Rectangle that represents the dimensions of the border.

   color: The System.Drawing.Color of the border.

   style: One of the System.Windows.Forms.ButtonBorderStyle values that specifies the style of the border.
  """
  pass
 @staticmethod
 def DrawBorder3D(graphics,*__args):
  """
  DrawBorder3D(graphics: Graphics,x: int,y: int,width: int,height: int)

   Draws a three-dimensional style border on the specified graphics surface and within the 

    specified bounds on a control.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the top left of the border rectangle.

   y: The y-coordinate of the top left of the border rectangle.

   width: The width of the border rectangle.

   height: The height of the border rectangle.

  DrawBorder3D(graphics: Graphics,x: int,y: int,width: int,height: int,style: Border3DStyle)

   Draws a three-dimensional style border with the specified style,on the specified graphics 

    surface,and within the specified bounds on a control.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the top left of the border rectangle.

   y: The y-coordinate of the top left of the border rectangle.

   width: The width of the border rectangle.

   height: The height of the border rectangle.

   style: One of the System.Windows.Forms.Border3DStyle values that specifies the style of the border.

  DrawBorder3D(graphics: Graphics,x: int,y: int,width: int,height: int,style: Border3DStyle,sides: Border3DSide)

   Draws a three-dimensional style border with the specified style,on the specified graphics 

    surface and side,and within the specified bounds on a control.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the top left of the border rectangle.

   y: The y-coordinate of the top left of the border rectangle.

   width: The width of the border rectangle.

   height: The height of the border rectangle.

   style: One of the System.Windows.Forms.Border3DStyle values that specifies the style of the border.

   sides: The System.Windows.Forms.Border3DSide of the rectangle to draw the border on.

  DrawBorder3D(graphics: Graphics,rectangle: Rectangle)

   Draws a three-dimensional style border on the specified graphics surface and within the 

    specified bounds on a control.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the border.

  DrawBorder3D(graphics: Graphics,rectangle: Rectangle,style: Border3DStyle)

   Draws a three-dimensional style border with the specified style,on the specified graphics 

    surface,and within the specified bounds on a control.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the border.

   style: One of the System.Windows.Forms.Border3DStyle values that specifies the style of the border.

  DrawBorder3D(graphics: Graphics,rectangle: Rectangle,style: Border3DStyle,sides: Border3DSide)

   Draws a three-dimensional style border with the specified style,on the specified graphics 

    surface and sides,and within the specified bounds on a control.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the border.

   style: One of the System.Windows.Forms.Border3DStyle values that specifies the style of the border.

   sides: One of the System.Windows.Forms.Border3DSide values that specifies the side of the rectangle to 

    draw the border on.
  """
  pass
 @staticmethod
 def DrawButton(graphics,*__args):
  """
  DrawButton(graphics: Graphics,x: int,y: int,width: int,height: int,state: ButtonState)

   Draws a button control in the specified state,on the specified graphics surface,and within the 

    specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the upper left corner of the drawing rectangle.

   y: The y-coordinate of the upper left corner of the drawing rectangle.

   width: The width of the button.

   height: The height of the button.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the button in.

  

  DrawButton(graphics: Graphics,rectangle: Rectangle,state: ButtonState)

   Draws a button control in the specified state,on the specified graphics surface,and within the 

    specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the button.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the button in.
  """
  pass
 @staticmethod
 def DrawCaptionButton(graphics,*__args):
  """
  DrawCaptionButton(graphics: Graphics,x: int,y: int,width: int,height: int,button: CaptionButton,state: ButtonState)

   Draws the specified caption button control in the specified state,on the specified graphics 

    surface,and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the top left of the drawing rectangle.

   y: The y-coordinate of the top left of the drawing rectangle.

   width: The width of the drawing rectangle.

   height: The height of the drawing rectangle.

   button: One of the System.Windows.Forms.CaptionButton values that specifies the type of caption button 

    to draw.

  

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the button in.

  

  DrawCaptionButton(graphics: Graphics,rectangle: Rectangle,button: CaptionButton,state: ButtonState)

   Draws the specified caption button control in the specified state,on the specified graphics 

    surface,and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the caption button.

   button: One of the System.Windows.Forms.CaptionButton values that specifies the type of caption button 

    to draw.

  

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the button in.
  """
  pass
 @staticmethod
 def DrawCheckBox(graphics,*__args):
  """
  DrawCheckBox(graphics: Graphics,rectangle: Rectangle,state: ButtonState)

   Draws a check box control in the specified state,on the specified graphics surface,and within 

    the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the check box.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the check box in.

  

  DrawCheckBox(graphics: Graphics,x: int,y: int,width: int,height: int,state: ButtonState)

   Draws a check box control in the specified state,on the specified graphics surface,and within 

    the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the upper left corner of the drawing rectangle.

   y: The y-coordinate of the upper left corner of the drawing rectangle.

   width: The width of the check box.

   height: The height of the check box.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the check box in.
  """
  pass
 @staticmethod
 def DrawComboButton(graphics,*__args):
  """
  DrawComboButton(graphics: Graphics,x: int,y: int,width: int,height: int,state: ButtonState)

   Draws a drop-down button on a combo box control in the specified state,on the specified 

    graphics surface,and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the top left of the border rectangle.

   y: The y-coordinate of the top left of the border rectangle.

   width: The width of the combo box.

   height: The height of the combo box.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the combo box in.

  

  DrawComboButton(graphics: Graphics,rectangle: Rectangle,state: ButtonState)

   Draws a drop-down button on a combo box control in the specified state,on the specified 

    graphics surface,and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the combo box.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the combo box in.
  """
  pass
 @staticmethod
 def DrawContainerGrabHandle(graphics,bounds):
  """
  DrawContainerGrabHandle(graphics: Graphics,bounds: Rectangle)

   Draws a container control grab handle glyph on the specified graphics surface and within the 

    specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   bounds: The System.Drawing.Rectangle that represents the dimensions of the grab handle glyph.
  """
  pass
 @staticmethod
 def DrawFocusRectangle(graphics,rectangle,foreColor=None,backColor=None):
  """
  DrawFocusRectangle(graphics: Graphics,rectangle: Rectangle,foreColor: Color,backColor: Color)

   Draws a focus rectangle on the specified graphics surface and within the specified bounds.

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the grab handle glyph.

   foreColor: The System.Drawing.Color that is the foreground color of the object to draw the focus rectangle 

    on.

  

   backColor: The System.Drawing.Color that is the background color of the object to draw the focus rectangle 

    on.

  

  DrawFocusRectangle(graphics: Graphics,rectangle: Rectangle)

   Draws a focus rectangle on the specified graphics surface and within the specified bounds.

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the grab handle glyph.
  """
  pass
 @staticmethod
 def DrawGrabHandle(graphics,rectangle,primary,enabled):
  """
  DrawGrabHandle(graphics: Graphics,rectangle: Rectangle,primary: bool,enabled: bool)

   Draws a standard selection grab handle glyph on the specified graphics surface,within the 

    specified bounds,and in the specified state and style.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the grab handle glyph.

   primary: true to draw the handle as a primary grab handle; otherwise,false.

   enabled: true to draw the handle in an enabled state; otherwise,false.
  """
  pass
 @staticmethod
 def DrawGrid(graphics,area,pixelsBetweenDots,backColor):
  """
  DrawGrid(graphics: Graphics,area: Rectangle,pixelsBetweenDots: Size,backColor: Color)

   Draws a grid of one-pixel dots with the specified spacing,within the specified bounds,on the 

    specified graphics surface,and in the specified color.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   area: The System.Drawing.Rectangle that represents the dimensions of the grid.

   pixelsBetweenDots: The System.Drawing.Size that specified the height and width between the dots of the grid.

   backColor: The System.Drawing.Color of the background behind the grid.
  """
  pass
 @staticmethod
 def DrawImageDisabled(graphics,image,x,y,background):
  """
  DrawImageDisabled(graphics: Graphics,image: Image,x: int,y: int,background: Color)

   Draws the specified image in a disabled state.

  

   graphics: The System.Drawing.Graphics to draw on.

   image: The System.Drawing.Image to draw.

   x: The x-coordinate of the top left of the border image.

   y: The y-coordinate of the top left of the border image.

   background: The System.Drawing.Color of the background behind the image.
  """
  pass
 @staticmethod
 def DrawLockedFrame(graphics,rectangle,primary):
  """
  DrawLockedFrame(graphics: Graphics,rectangle: Rectangle,primary: bool)

   Draws a locked selection frame on the screen within the specified bounds and on the specified 

    graphics surface. Specifies whether to draw the frame with the primary selected colors.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the frame.

   primary: true to draw this frame with the colors used for the primary selection; otherwise,false.
  """
  pass
 @staticmethod
 def DrawMenuGlyph(graphics,*__args):
  """
  DrawMenuGlyph(graphics: Graphics,x: int,y: int,width: int,height: int,glyph: MenuGlyph)

   Draws the specified menu glyph on a menu item control with the specified bounds and on the 

    specified surface.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the upper left corner of the drawing rectangle.

   y: The y-coordinate of the upper left corner of the drawing rectangle.

   width: The width of the menu glyph.

   height: The height of the menu glyph.

   glyph: One of the System.Windows.Forms.MenuGlyph values that specifies the image to draw.

  DrawMenuGlyph(graphics: Graphics,x: int,y: int,width: int,height: int,glyph: MenuGlyph,foreColor: Color,backColor: Color)

   Draws the specified menu glyph on a menu item control within the specified coordinates,height,

    and width on the specified surface,replacing System.Drawing.Color.White with the color 

    specified in the backColor parameter and replacing System.Drawing.Color.Black with the color 

    specified in the foreColor parameter.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the upper left corner of the drawing rectangle.

   y: The y-coordinate of the upper left corner of the drawing rectangle.

   width: The width of the menu glyph.

   height: The height of the menu glyph.

   glyph: One of the System.Windows.Forms.MenuGlyph values that specifies the image to draw.

   foreColor: The color that replaces System.Drawing.Color.White as the foreground color.

   backColor: The color that replaces System.Drawing.Color.Black as the background color.

  DrawMenuGlyph(graphics: Graphics,rectangle: Rectangle,glyph: MenuGlyph)

   Draws the specified menu glyph on a menu item control within the specified bounds and on the 

    specified surface.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the glyph.

   glyph: One of the System.Windows.Forms.MenuGlyph values that specifies the image to draw.

  DrawMenuGlyph(graphics: Graphics,rectangle: Rectangle,glyph: MenuGlyph,foreColor: Color,backColor: Color)

   Draws the specified menu glyph on a menu item control within the specified bounds and on the 

    specified surface,replacing System.Drawing.Color.White with the color specified in the 

    backColor parameter and replacing System.Drawing.Color.Black with the color specified in the 

    foreColor parameter.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the glyph.

   glyph: One of the System.Windows.Forms.MenuGlyph values that specifies the image to draw.

   foreColor: The color that replaces System.Drawing.Color.White as the foreground color.

   backColor: The color that replaces System.Drawing.Color.Black as the background color.
  """
  pass
 @staticmethod
 def DrawMixedCheckBox(graphics,*__args):
  """
  DrawMixedCheckBox(graphics: Graphics,x: int,y: int,width: int,height: int,state: ButtonState)

   Draws a three-state check box control in the specified state,on the specified graphics surface,

    and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the upper left corner of the drawing rectangle.

   y: The y-coordinate of the upper left corner of the drawing rectangle.

   width: The width of the check box.

   height: The height of the check box.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the check box in.

  

  DrawMixedCheckBox(graphics: Graphics,rectangle: Rectangle,state: ButtonState)

   Draws a three-state check box control in the specified state,on the specified graphics surface,

    and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the check box.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the check box in.
  """
  pass
 @staticmethod
 def DrawRadioButton(graphics,*__args):
  """
  DrawRadioButton(graphics: Graphics,rectangle: Rectangle,state: ButtonState)

   Draws a radio button control in the specified state,on the specified graphics surface,and 

    within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the radio button.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the radio button in.

  

  DrawRadioButton(graphics: Graphics,x: int,y: int,width: int,height: int,state: ButtonState)

   Draws a radio button control in the specified state,on the specified graphics surface,and 

    within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the upper left corner of the drawing rectangle.

   y: The y-coordinate of the upper left corner of the drawing rectangle.

   width: The width of the radio button.

   height: The height of the radio button.

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the radio button in.
  """
  pass
 @staticmethod
 def DrawReversibleFrame(rectangle,backColor,style):
  """
  DrawReversibleFrame(rectangle: Rectangle,backColor: Color,style: FrameStyle)

   Draws a reversible frame on the screen within the specified bounds,with the specified 

    background color,and in the specified state.

  

  

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the rectangle to draw,in screen 

    coordinates.

  

   backColor: The System.Drawing.Color of the background behind the frame.

   style: One of the System.Windows.Forms.FrameStyle values that specifies the style of the frame.
  """
  pass
 @staticmethod
 def DrawReversibleLine(start,end,backColor):
  """
  DrawReversibleLine(start: Point,end: Point,backColor: Color)

   Draws a reversible line on the screen within the specified starting and ending points and with 

    the specified background color.

  

  

   start: The starting System.Drawing.Point of the line,in screen coordinates.

   end: The ending System.Drawing.Point of the line,in screen coordinates.

   backColor: The System.Drawing.Color of the background behind the line.
  """
  pass
 @staticmethod
 def DrawScrollButton(graphics,*__args):
  """
  DrawScrollButton(graphics: Graphics,x: int,y: int,width: int,height: int,button: ScrollButton,state: ButtonState)

   Draws the specified scroll button on a scroll bar control in the specified state,on the 

    specified graphics surface,and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   x: The x-coordinate of the upper left corner of the drawing rectangle.

   y: The y-coordinate of the upper left corner of the drawing rectangle.

   width: The width of the scroll button.

   height: The height of the scroll button.

   button: One of the System.Windows.Forms.ScrollButton values that specifies the type of scroll arrow to 

    draw.

  

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the scroll button in.

  

  DrawScrollButton(graphics: Graphics,rectangle: Rectangle,button: ScrollButton,state: ButtonState)

   Draws the specified scroll button on a scroll bar control in the specified state,on the 

    specified graphics surface,and within the specified bounds.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the glyph.

   button: One of the System.Windows.Forms.ScrollButton values that specifies the type of scroll arrow to 

    draw.

  

   state: A bitwise combination of the System.Windows.Forms.ButtonState values that specifies the state to 

    draw the scroll button in.
  """
  pass
 @staticmethod
 def DrawSelectionFrame(graphics,active,outsideRect,insideRect,backColor):
  """
  DrawSelectionFrame(graphics: Graphics,active: bool,outsideRect: Rectangle,insideRect: Rectangle,backColor: Color)

   Draws a standard selection frame in the specified state,on the specified graphics surface,with 

    the specified inner and outer dimensions,and with the specified background color.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   active: true to draw the selection frame in an active state; otherwise,false.

   outsideRect: The System.Drawing.Rectangle that represents the outer boundary of the selection frame.

   insideRect: The System.Drawing.Rectangle that represents the inner boundary of the selection frame.

   backColor: The System.Drawing.Color of the background behind the frame.
  """
  pass
 @staticmethod
 def DrawSizeGrip(graphics,backColor,*__args):
  """
  DrawSizeGrip(graphics: Graphics,backColor: Color,x: int,y: int,width: int,height: int)

   Draws a size grip on a form with the specified bounds and background color and on the specified 

    graphics surface.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   backColor: The System.Drawing.Color of the background used to determine the colors of the size grip.

   x: The x-coordinate of the upper left corner of the size grip.

   y: The y-coordinate of the upper left corner of the size grip.

   width: The width of the size grip.

   height: The height of the size grip.

  DrawSizeGrip(graphics: Graphics,backColor: Color,bounds: Rectangle)

   Draws a size grip on a form with the specified bounds and background color and on the specified 

    graphics surface.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   backColor: The System.Drawing.Color of the background used to determine the colors of the size grip.

   bounds: The System.Drawing.Rectangle that represents the dimensions of the size grip.
  """
  pass
 @staticmethod
 def DrawStringDisabled(*__args):
  """
  DrawStringDisabled(dc: IDeviceContext,s: str,font: Font,color: Color,layoutRectangle: Rectangle,format: TextFormatFlags)

   Draws the specified string in a disabled state on the specified graphics surface,within the 

    specified bounds,and in the specified font,color,and format,using the specified GDI-based 

    System.Windows.Forms.TextRenderer.

  

  

   dc: The GDI-based System.Windows.Forms.TextRenderer.

   s: The string to draw.

   font: The System.Drawing.Font to draw the string with.

   color: The System.Drawing.Color of the background behind the string.

   layoutRectangle: The System.Drawing.RectangleF that represents the dimensions of the string.

   format: The System.Drawing.StringFormat to apply to the string.

  DrawStringDisabled(graphics: Graphics,s: str,font: Font,color: Color,layoutRectangle: RectangleF,format: StringFormat)

   Draws the specified string in a disabled state on the specified graphics surface; within the 

    specified bounds; and in the specified font,color,and format.

  

  

   graphics: The System.Drawing.Graphics to draw on.

   s: The string to draw.

   font: The System.Drawing.Font to draw the string with.

   color: The System.Drawing.Color of the background behind the string.

   layoutRectangle: The System.Drawing.RectangleF that represents the dimensions of the string.

   format: The System.Drawing.StringFormat to apply to the string.
  """
  pass
 @staticmethod
 def DrawVisualStyleBorder(graphics,bounds):
  """
  DrawVisualStyleBorder(graphics: Graphics,bounds: Rectangle)

   Draws a border in the style appropriate for disabled items.

  

   graphics: The System.Drawing.Graphics to draw on.

   bounds: The System.Drawing.Rectangle that represents the dimensions of the border.
  """
  pass
 @staticmethod
 def FillReversibleRectangle(rectangle,backColor):
  """
  FillReversibleRectangle(rectangle: Rectangle,backColor: Color)

   Draws a filled,reversible rectangle on the screen.

  

   rectangle: The System.Drawing.Rectangle that represents the dimensions of the rectangle to fill,in screen 

    coordinates.

  

   backColor: The System.Drawing.Color of the background behind the fill.
  """
  pass
 @staticmethod
 def Light(baseColor,percOfLightLight=None):
  """
  Light(baseColor: Color) -> Color

  

   Creates a new light color object for the control from the specified color.

  

   baseColor: The System.Drawing.Color to be lightened.

   Returns: A System.Drawing.Color that represents the light color on the control.

  Light(baseColor: Color,percOfLightLight: Single) -> Color

  

   Creates a new light color object for the control from the specified color and lightens it by the 

    specified percentage.

  

  

   baseColor: The System.Drawing.Color to be lightened.

   percOfLightLight: The percentage to lighten the specified System.Drawing.Color.

   Returns: A System.Drawing.Color that represents the light color on the control.
  """
  pass
 @staticmethod
 def LightLight(baseColor):
  """
  LightLight(baseColor: Color) -> Color

  

   Creates a new light color object for the control from the specified color.

  

   baseColor: The System.Drawing.Color to be lightened.

   Returns: A System.Drawing.Color that represents the light color on the control.
  """
  pass
 ContrastControlDark=None




class CryptoKeyAuditRule(AuditRule):
 """
 Represents an audit rule for a cryptographic key. An audit rule represents a combination of a user's identity and an access mask. An audit rule also contains information about the how the rule is inherited by child objects,how that inheritance is propagated,and for what conditions it is audited.

 

 CryptoKeyAuditRule(identity: IdentityReference,cryptoKeyRights: CryptoKeyRights,flags: AuditFlags)

 CryptoKeyAuditRule(identity: str,cryptoKeyRights: CryptoKeyRights,flags: AuditFlags)
 """
 @staticmethod
 def __new__(self,identity,cryptoKeyRights,flags):
  """
  __new__(cls: type,identity: IdentityReference,cryptoKeyRights: CryptoKeyRights,flags: AuditFlags)

  __new__(cls: type,identity: str,cryptoKeyRights: CryptoKeyRights,flags: AuditFlags)
  """
  pass
 AccessMask=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the access mask for this rule.



"""

 CryptoKeyRights=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the cryptographic key operation for which this audit rule generates audits.



Get: CryptoKeyRights(self: CryptoKeyAuditRule) -> CryptoKeyRights



"""





class ListViewHitTestLocations(Enum,IComparable,IFormattable,IConvertible):
 """
 Defines constants that represent areas in a System.Windows.Forms.ListView or System.Windows.Forms.ListViewItem.

 

 enum (flags) ListViewHitTestLocations,values: AboveClientArea (256),BelowClientArea (16),Image (2),Label (4),LeftOfClientArea (64),None (1),RightOfClientArea (32),StateImage (512)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AboveClientArea=None
 BelowClientArea=None
 Image=None
 Label=None
 LeftOfClientArea=None
 None=None
 RightOfClientArea=None
 StateImage=None
 value__=None




class ProcessModule(Component,IComponent,IDisposable):
 """ Represents a.dll or .exe file that is loaded into a particular process. """
 def Dispose(self):
  """
  Dispose(self: Component,disposing: bool)

   Releases the unmanaged resources used by the System.ComponentModel.Component and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def ToString(self):
  """
  ToString(self: ProcessModule) -> str

  

   Converts the name of the module to a string.

   Returns: The value of the System.Diagnostics.ProcessModule.ModuleName property.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass
 BaseAddress=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the memory address where the module was loaded.



Get: BaseAddress(self: ProcessModule) -> IntPtr



"""

 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 EntryPointAddress=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the memory address for the function that runs when the system loads and runs the module.



Get: EntryPointAddress(self: ProcessModule) -> IntPtr



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 FileName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the full path to the module.



Get: FileName(self: ProcessModule) -> str



"""

 FileVersionInfo=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets version information about the module.



Get: FileVersionInfo(self: ProcessModule) -> FileVersionInfo



"""

 ModuleMemorySize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the amount of memory that is required to load the module.



Get: ModuleMemorySize(self: ProcessModule) -> int



"""

 ModuleName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the process module.



Get: ModuleName(self: ProcessModule) -> str



"""





class TreeViewImageKeyConverter(ImageKeyConverter):
 """
 Provides a type converter to convert data for an image key to and from another data type.

 

 TreeViewImageKeyConverter()
 """
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: TreeViewImageKeyConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the specified object to an object of the specified type using the specified culture 

    information and context.

  

  

   context: A System.ComponentModel.ITypeDescriptorContext that provides a format context,which can be used 

    to extract additional information about the environment this type converter is being invoked 

    from. This parameter or properties of this parameter can be null.

  

   culture: A System.Globalization.CultureInfo that provides locale information.

   value: The object to convert,typically an image key.

   destinationType: The type to convert the object to.

   Returns: An System.Object that represents the converted value.
  """
  pass
 IncludeNoneAsStandardValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether null is valid in the System.ComponentModel.TypeConverter.StandardValuesCollection collection.



"""





class HttpVersion(object):
 """
 Defines the HTTP version numbers that are supported by the System.Net.HttpWebRequest and System.Net.HttpWebResponse classes.

 

 HttpVersion()
 """
 Version10=None
 Version11=None




class SecurityIDType(Enum,IComparable,IFormattable,IConvertible):
 """
 This enumeration is not used.

 

 enum SecurityIDType,values: Alias (4),Computer (9),DeletedAccount (6),Domain (3),Group (2),Invalid (7),Unknown (8),User (1),WellKnownGroup (5)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Alias=None
 Computer=None
 DeletedAccount=None
 Domain=None
 Group=None
 Invalid=None
 Unknown=None
 User=None
 value__=None
 WellKnownGroup=None




class DataGridRowClipboardEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Controls.DataGrid.CopyingRowClipboardContent event.

 

 DataGridRowClipboardEventArgs(item: object,startColumnDisplayIndex: int,endColumnDisplayIndex: int,isColumnHeadersRow: bool)
 """
 def FormatClipboardCellValues(self,format):
  """
  FormatClipboardCellValues(self: DataGridRowClipboardEventArgs,format: str) -> str

  

   Returns the System.Windows.Controls.DataGridRowClipboardEventArgs.ClipboardRowContent values as 

    a string in the specified format.

  

  

   format: The data format in which to serialize the cell values.

   Returns: The formatted string.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 @staticmethod
 def __new__(self,item,startColumnDisplayIndex,endColumnDisplayIndex,isColumnHeadersRow):
  """ __new__(cls: type,item: object,startColumnDisplayIndex: int,endColumnDisplayIndex: int,isColumnHeadersRow: bool) """
  pass
 ClipboardRowContent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a list of System.Windows.Controls.DataGridClipboardCellContent values that represent the text values of the cells being copied.



Get: ClipboardRowContent(self: DataGridRowClipboardEventArgs) -> List[DataGridClipboardCellContent]



"""

 EndColumnDisplayIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Controls.DataGridColumn.DisplayIndex value of the column that contains the last selected cell in the row.



Get: EndColumnDisplayIndex(self: DataGridRowClipboardEventArgs) -> int



"""

 IsColumnHeadersRow=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the row for which the event occurred represents the column headers.



Get: IsColumnHeadersRow(self: DataGridRowClipboardEventArgs) -> bool



"""

 Item=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the data item for the row for which the event occurred.



Get: Item(self: DataGridRowClipboardEventArgs) -> object



"""

 StartColumnDisplayIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Controls.DataGridColumn.DisplayIndex value of the column that contains the first selected cell in the row.



Get: StartColumnDisplayIndex(self: DataGridRowClipboardEventArgs) -> int



"""





class CopyPixelOperation(Enum,IComparable,IFormattable,IConvertible):
 """
 Determines how the source color in a copy pixel operation is combined with the destination color to result in a final color.

 

 enum CopyPixelOperation,values: Blackness (66),CaptureBlt (1073741824),DestinationInvert (5570569),MergeCopy (12583114),MergePaint (12255782),NoMirrorBitmap (-2147483648),NotSourceCopy (3342344),NotSourceErase (1114278),PatCopy (15728673),PatInvert (5898313),PatPaint (16452105),SourceAnd (8913094),SourceCopy (13369376),SourceErase (4457256),SourceInvert (6684742),SourcePaint (15597702),Whiteness (16711778)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Blackness=None
 CaptureBlt=None
 DestinationInvert=None
 MergeCopy=None
 MergePaint=None
 NoMirrorBitmap=None
 NotSourceCopy=None
 NotSourceErase=None
 PatCopy=None
 PatInvert=None
 PatPaint=None
 SourceAnd=None
 SourceCopy=None
 SourceErase=None
 SourceInvert=None
 SourcePaint=None
 value__=None
 Whiteness=None




class IRaiseItemChangedEvents:
 """ Indicates whether a class converts property change events to System.ComponentModel.IBindingList.ListChanged events. """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 RaisesItemChangedEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the System.ComponentModel.IRaiseItemChangedEvents object raises System.ComponentModel.IBindingList.ListChanged events.



Get: RaisesItemChangedEvents(self: IRaiseItemChangedEvents) -> bool



"""





class ToolStripItemCollection(ArrangedElementCollection,IList,ICollection,IEnumerable):
 """
 Represents a collection of System.Windows.Forms.ToolStripItem objects.

 

 ToolStripItemCollection(owner: ToolStrip,value: Array[ToolStripItem])
 """
 def Add(self,*__args):
  """
  Add(self: ToolStripItemCollection,text: str,image: Image,onClick: EventHandler) -> ToolStripItem

  

   Adds a System.Windows.Forms.ToolStripItem that displays the specified image and text to the 

    collection and that raises the System.Windows.Forms.ToolStripItem.Click event.

  

  

   text: The text to be displayed on the System.Windows.Forms.ToolStripItem.

   image: The System.Drawing.Image to be displayed on the System.Windows.Forms.ToolStripItem.

   onClick: Raises the System.Windows.Forms.ToolStripItem.Click event.

   Returns: The new System.Windows.Forms.ToolStripItem.

  Add(self: ToolStripItemCollection,value: ToolStripItem) -> int

  

   Adds the specified item to the end of the collection.

  

   value: The System.Windows.Forms.ToolStripItem to add to the end of the collection.

   Returns: An System.Int32 representing the zero-based index of the new item in the collection.

  Add(self: ToolStripItemCollection,text: str,image: Image) -> ToolStripItem

  

   Adds a System.Windows.Forms.ToolStripItem that displays the specified image and text to the 

    collection.

  

  

   text: The text to be displayed on the System.Windows.Forms.ToolStripItem.

   image: The System.Drawing.Image to be displayed on the System.Windows.Forms.ToolStripItem.

   Returns: The new System.Windows.Forms.ToolStripItem.

  Add(self: ToolStripItemCollection,text: str) -> ToolStripItem

  

   Adds a System.Windows.Forms.ToolStripItem that displays the specified text to the collection.

  

   text: The text to be displayed on the System.Windows.Forms.ToolStripItem.

   Returns: The new System.Windows.Forms.ToolStripItem.

  Add(self: ToolStripItemCollection,image: Image) -> ToolStripItem

  

   Adds a System.Windows.Forms.ToolStripItem that displays the specified image to the collection.

  

   image: The System.Drawing.Image to be displayed on the System.Windows.Forms.ToolStripItem.

   Returns: The new System.Windows.Forms.ToolStripItem.
  """
  pass
 def AddRange(self,toolStripItems):
  """
  AddRange(self: ToolStripItemCollection,toolStripItems: ToolStripItemCollection)

   Adds a System.Windows.Forms.ToolStripItemCollection to the current collection.

  

   toolStripItems: The System.Windows.Forms.ToolStripItemCollection to be added to the current collection.

  AddRange(self: ToolStripItemCollection,toolStripItems: Array[ToolStripItem])

   Adds an array of System.Windows.Forms.ToolStripItem controls to the collection.

  

   toolStripItems: An array of System.Windows.Forms.ToolStripItem controls.
  """
  pass
 def Clear(self):
  """
  Clear(self: ToolStripItemCollection)

   Removes all items from the collection.
  """
  pass
 def Contains(self,value):
  """
  Contains(self: ToolStripItemCollection,value: ToolStripItem) -> bool

  

   Determines whether the specified item is a member of the collection.

  

   value: The System.Windows.Forms.ToolStripItem to search for in the 

    System.Windows.Forms.ToolStripItemCollection.

  

   Returns: true if the System.Windows.Forms.ToolStripItem is a member of the current 

    System.Windows.Forms.ToolStripItemCollection; otherwise,false.
  """
  pass
 def ContainsKey(self,key):
  """
  ContainsKey(self: ToolStripItemCollection,key: str) -> bool

  

   Determines whether the collection contains an item with the specified key.

  

   key: The key to locate in the System.Windows.Forms.ToolStripItemCollection.

   Returns: true if the System.Windows.Forms.ToolStripItemCollection contains a 

    System.Windows.Forms.ToolStripItem with the specified key; otherwise,false.
  """
  pass
 def CopyTo(self,array,index):
  """
  CopyTo(self: ToolStripItemCollection,array: Array[ToolStripItem],index: int)

   Copies the collection into the specified position of the specified 

    System.Windows.Forms.ToolStripItem array.

  

  

   array: The array of type System.Windows.Forms.ToolStripItem to which to copy the collection.

   index: The position in the System.Windows.Forms.ToolStripItem array at which to paste the collection.
  """
  pass
 def Find(self,key,searchAllChildren):
  """
  Find(self: ToolStripItemCollection,key: str,searchAllChildren: bool) -> Array[ToolStripItem]

  

   Searches for items by their name and returns an array of all matching controls.

  

   key: The item name to search the System.Windows.Forms.ToolStripItemCollection for.

   searchAllChildren: true to search child items of the System.Windows.Forms.ToolStripItem specified by the key 

    parameter; otherwise,false.

  

   Returns: A System.Windows.Forms.ToolStripItem array of the search results.
  """
  pass
 def IndexOf(self,value):
  """
  IndexOf(self: ToolStripItemCollection,value: ToolStripItem) -> int

  

   Retrieves the index of the specified item in the collection.

  

   value: The System.Windows.Forms.ToolStripItem to locate in the 

    System.Windows.Forms.ToolStripItemCollection.

  

   Returns: A zero-based index value that represents the position of the specified 

    System.Windows.Forms.ToolStripItem in the System.Windows.Forms.ToolStripItemCollection,if 

    found; otherwise,-1.
  """
  pass
 def IndexOfKey(self,key):
  """
  IndexOfKey(self: ToolStripItemCollection,key: str) -> int

  

   Retrieves the index of the first occurrence of the specified item within the collection.

  

   key: The name of the System.Windows.Forms.ToolStripItem to search for.

   Returns: A zero-based index value that represents the position of the first occurrence of the 

    System.Windows.Forms.ToolStripItem specified by the key parameter,if found; otherwise,-1.
  """
  pass
 def Insert(self,index,value):
  """
  Insert(self: ToolStripItemCollection,index: int,value: ToolStripItem)

   Inserts the specified item into the collection at the specified index.

  

   index: The location in the System.Windows.Forms.ToolStripItemCollection at which to insert the 

    System.Windows.Forms.ToolStripItem.

  

   value: The System.Windows.Forms.ToolStripItem to insert.
  """
  pass
 def Remove(self,value):
  """
  Remove(self: ToolStripItemCollection,value: ToolStripItem)

   Removes the specified item from the collection.

  

   value: The System.Windows.Forms.ToolStripItem to remove from the 

    System.Windows.Forms.ToolStripItemCollection.
  """
  pass
 def RemoveAt(self,index):
  """
  RemoveAt(self: ToolStripItemCollection,index: int)

   Removes an item from the specified index in the collection.

  

   index: The index value of the System.Windows.Forms.ToolStripItem to remove.
  """
  pass
 def RemoveByKey(self,key):
  """
  RemoveByKey(self: ToolStripItemCollection,key: str)

   Removes the item that has the specified key.

  

   key: The key of the System.Windows.Forms.ToolStripItem to remove.
  """
  pass
 def __add__(self,*args):
  """ x.__add__(y) <==> x+yx.__add__(y) <==> x+yx.__add__(y) <==> x+yx.__add__(y) <==> x+yx.__add__(y) <==> x+y """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: IList,value: object) -> bool

  

   Determines whether the System.Collections.IList contains a specific value.

  

   value: The object to locate in the System.Collections.IList.

   Returns: true if the System.Object is found in the System.Collections.IList; otherwise,false.
  """
  pass
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y]x.__getitem__(y) <==> x[y] """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 @staticmethod
 def __new__(self,owner,value):
  """ __new__(cls: type,owner: ToolStrip,value: Array[ToolStripItem]) """
  pass
 IsReadOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the System.Windows.Forms.ToolStripItemCollection is read-only.



Get: IsReadOnly(self: ToolStripItemCollection) -> bool



"""





class OwnerDrawPropertyBag(MarshalByRefObject,ISerializable):
 """ Contains values of properties that a component might need only occasionally. """
 @staticmethod
 def Copy(value):
  """
  Copy(value: OwnerDrawPropertyBag) -> OwnerDrawPropertyBag

  

   Copies an System.Windows.Forms.OwnerDrawPropertyBag.

  

   value: The System.Windows.Forms.OwnerDrawPropertyBag to be copied.

   Returns: A new System.Windows.Forms.OwnerDrawPropertyBag.
  """
  pass
 def IsEmpty(self):
  """
  IsEmpty(self: OwnerDrawPropertyBag) -> bool

  

   Returns whether the System.Windows.Forms.OwnerDrawPropertyBag contains all default values.

   Returns: true if the System.Windows.Forms.OwnerDrawPropertyBag contains all default values; otherwise,

    false.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """ __new__(cls: type,info: SerializationInfo,context: StreamingContext) """
  pass
 def __reduce_ex__(self,*args):
  pass
 BackColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the background color for the component.



Get: BackColor(self: OwnerDrawPropertyBag) -> Color



Set: BackColor(self: OwnerDrawPropertyBag)=value

"""

 Font=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the font of the text displayed by the component.



Get: Font(self: OwnerDrawPropertyBag) -> Font



Set: Font(self: OwnerDrawPropertyBag)=value

"""

 ForeColor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the foreground color of the component.



Get: ForeColor(self: OwnerDrawPropertyBag) -> Color



Set: ForeColor(self: OwnerDrawPropertyBag)=value

"""





class AttributeTargets(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the application elements on which it is valid to apply an attribute.

 

 enum (flags) AttributeTargets,values: All (32767),Assembly (1),Class (4),Constructor (32),Delegate (4096),Enum (16),Event (512),Field (256),GenericParameter (16384),Interface (1024),Method (64),Module (2),Parameter (2048),Property (128),ReturnValue (8192),Struct (8)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 All=None
 Assembly=None
 Class=None
 Constructor=None
 Delegate=None
 Enum=None
 Event=None
 Field=None
 GenericParameter=None
 Interface=None
 Method=None
 Module=None
 Parameter=None
 Property=None
 ReturnValue=None
 Struct=None
 value__=None




class BooleanToVisibilityConverter(object,IValueConverter):
 """
 Represents the converter that converts Boolean values to and from System.Windows.Visibility enumeration values.

 

 BooleanToVisibilityConverter()
 """
 def Convert(self,value,targetType,parameter,culture):
  """
  Convert(self: BooleanToVisibilityConverter,value: object,targetType: Type,parameter: object,culture: CultureInfo) -> object

  

   Converts a Boolean value to a System.Windows.Visibility enumeration value.

  

   value: The Boolean value to convert. This value can be a standard Boolean value or a nullable Boolean 

    value.

  

   targetType: This parameter is not used.

   parameter: This parameter is not used.

   culture: This parameter is not used.

   Returns: System.Windows.Visibility.Visible if value is true; otherwise,

    System.Windows.Visibility.Collapsed.
  """
  pass
 def ConvertBack(self,value,targetType,parameter,culture):
  """
  ConvertBack(self: BooleanToVisibilityConverter,value: object,targetType: Type,parameter: object,culture: CultureInfo) -> object

  

   Converts a System.Windows.Visibility enumeration value to a Boolean value.

  

   value: A System.Windows.Visibility enumeration value.

   targetType: This parameter is not used.

   parameter: This parameter is not used.

   culture: This parameter is not used.

   Returns: true if value is System.Windows.Visibility.Visible; otherwise,false.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass



class DefaultValueAttribute(Attribute,_Attribute):
 """
 Specifies the default value for a property.

 

 DefaultValueAttribute(type: Type,value: str)

 DefaultValueAttribute(value: Char)

 DefaultValueAttribute(value: Byte)

 DefaultValueAttribute(value: Int16)

 DefaultValueAttribute(value: int)

 DefaultValueAttribute(value: Int64)

 DefaultValueAttribute(value: Single)

 DefaultValueAttribute(value: float)

 DefaultValueAttribute(value: bool)

 DefaultValueAttribute(value: str)

 DefaultValueAttribute(value: object)
 """
 def Equals(self,obj):
  """
  Equals(self: DefaultValueAttribute,obj: object) -> bool

  

   Returns whether the value of the given object is equal to the current 

    System.ComponentModel.DefaultValueAttribute.

  

  

   obj: The object to test the value equality of.

   Returns: true if the value of the given object is equal to that of the current; otherwise,false.
  """
  pass
 def GetHashCode(self):
  """ GetHashCode(self: DefaultValueAttribute) -> int """
  pass
 def SetValue(self,*args):
  """
  SetValue(self: DefaultValueAttribute,value: object)

   Sets the default value for the property to which this attribute is bound.

  

   value: The default value.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type,type: Type,value: str)

  __new__(cls: type,value: Char)

  __new__(cls: type,value: Byte)

  __new__(cls: type,value: Int16)

  __new__(cls: type,value: int)

  __new__(cls: type,value: Int64)

  __new__(cls: type,value: Single)

  __new__(cls: type,value: float)

  __new__(cls: type,value: bool)

  __new__(cls: type,value: str)

  __new__(cls: type,value: object)
  """
  pass
 def __ne__(self,*args):
  pass
 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the default value of the property this attribute is bound to.



Get: Value(self: DefaultValueAttribute) -> object



"""


